c
c-----------------------------------------------------------------------
c
      subroutine cylinder (
     &  x0, y0, z0,   !Coordinates of first end point
     &  x1, y1, z1,   !Coordinates of the other end
     &  radius,       !desired cylinder radius
     &  n,            !number of points in base
     &  x, y, z,      !coordinates of base points
     &  nv,           !number of vertices created (= 2*n)
     &  nf,           !number of faces (= n)
     &  nvf,          !number of vertex per face (= 4)
     &  mvf,          !max. allowed number of vertices per face
     &  ifa           !index of vertices in a face
     &  )
c
c.....cylinder - returns the coordinates of a rect prism of n sides
c     that approximates a cylinder from the point (x0,y0,z0) to the
c     point (x1,y1,z1).
c
      implicit none
      include           'consts.inc'
      real*8            x0, y0, z0, x1, y1, z1, x(*), y(*), z(*), radius
      integer           n, nv, nf, nvf, mvf, ifa(mvf,*)
c
      real*8            r, theta, phi, sp, cp, st, ct, rot(3,3),
     &                  xx, yy, zz, alpha
      integer           i, i2, i3, i4
c
c.....Use (x0,y0,z0) as the origin of coordinates and determine the
c     polar coordinates of the other cylinder's end:
c
      r = sqrt ((x1-x0)**2 + (y1-y0)**2 + (z1-z0)**2)
      theta = acos ((z1-z0)/r)
      phi = atan2 ((y1-y0), (x1-x0))
      ct = cos(theta)
      st = sin(theta)
      cp = cos(phi)
      sp = sin(phi)
c
c.....Get the coordinates of a prism oriented along the z axis:
c
      do i = 1, n
         alpha = (i-1) * 2d0 * pi / n
         x(i) = radius * cos(alpha)
         y(i) = radius * sin(alpha)
         z(i) = 0d0
         x(i+n) = x(i)
         y(i+n) = y(i)
         z(i+n) = r
      enddo
c
c.....rotate and translate the prims to the final orientation:
c
      rot(1,1) = cp*ct
      rot(1,2) = -sp
      rot(1,3) = cp*st
      rot(2,1) = sp*ct
      rot(2,2) = cp
      rot(2,3) = sp*st
      rot(3,1) = -st
      rot(3,2) = 0d0
      rot(3,3) = ct
      do i = 1, n + n
         xx = x(i)
         yy = y(i)
         zz = z(i)
         x(i) = x0 + rot(1,1) * xx + rot(1,2) * yy + rot(1,3) * zz
         y(i) = y0 + rot(2,1) * xx + rot(2,2) * yy + rot(2,3) * zz
         z(i) = z0 + rot(3,1) * xx + rot(3,2) * yy + rot(3,3) * zz
      enddo
      nv = n + n
c
c.....create the faces:
c
      nvf = 4
      nf = n
      do i = 1, n
         i2 = i + 1
         if (i .eq. n) i2 = 1
         i3 = i2 + n
         i4 = i + n
         ifa(1,i) = i
         ifa(2,i) = i2
         ifa(3,i) = i3
         ifa(4,i) = i4
      enddo
c
      end
