c
c-----------------------------------------------------------------------
c
      subroutine domisc (uin, uout)
c
c.....domisc - read-and-plot modes that do not fit naturally on the
c     tesselation or parametric packages.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'strtools.inc'
c
      integer           uin, uout
c
c.....Polyhedra data structure:
c
      include           'polyhedron.inc'
c
c.....D3D data:
c
      integer           D3DMPT
      parameter         (D3DMPT = 10201)
      integer           lud3d
      integer           d3dnx, d3dny
      real*8            d3dxmin, d3dxmax, d3dymin, d3dymax, d3dz(D3DMPT)
      real*8            d3dzmin, d3dzmax, d3dxstep, d3dystep
      real*8            fmin, fmax
      logical           d3dzclip
      integer           zuse
      integer           zuse_height, zuse_color, zuse_both
      parameter         (zuse_height = 1, zuse_color = 2, zuse_both = 3)
c
      integer           cook_linear, cook_atan, cook_tanh, cook_log
      parameter         (cook_linear = 1, cook_atan = 2, cook_tanh = 3,
     &                  cook_log = 4)
      real*8            cookfactor
      integer           cook_method
c
c.....3DLINES data:
c
      integer           con_mode
      integer           con_dots, con_cylinders, con_both
      parameter         (con_dots=10, con_cylinders=11, con_both=12)
c
      integer           lu3dl
      real*8            defrgb(4), deflinerad, defpointrad
c
      integer           i, j, ipt0, i1, j1, j2, i1j1, i1j2, i2j1, i2j2
      integer           ndeg, lp, lf, lw
      real*8            xx, yy, zz, tt
      character*(600)   line
      character*(80)    word
      character*(80)    file, filepov, fileoff, filevrml
      logical           done, done1, dopov, dooff, dovrml, ok
c
      integer           leng, fopen
      logical           precompile, fgetvarname, expression, finputline
      logical           setdble, setint
      character*(80)    uppcase, getword
c
c.....Define default values:
c
      dooff = .false.
      dopov = .false.
      dovrml = .false.
      write (uout,800)
c
c.....Read and interpret the next input file order:
c
      done = .false.
      do while (.not. done)
         ok = finputline (uin, line)
         if (.not. ok) call error ('parametric',
     &                 'Error looking for next input line', faterr)
         lp = 1
         word = getword (word, line, lp)
         word = uppcase (word)
         lw = leng(word)
c
c.......................................................................
         if (word(1:lw) .eq. 'READD3D') then
c.......................................................................
c
c...........read in the name of the D3D data file:
c
            file = getword (file, line, lp)
            lf = leng(file)
            write (uout,810) file(1:lf)
c
c...........read in D3D data file:
c
            if (fopen (lud3d, file, ioread) .eq. ioerror) then
               call error ('domisc', 'I/O error opening D3D file',
     &              warning)
               return
            endif 
            read (lud3d,'(a)') word
            read (lud3d,'(a)') word
            read (lud3d,'(a)') word
            read (lud3d,'(a)') word
            read (lud3d,*) d3dxmin, d3dxmax, d3dnx,
     &                     d3dymin, d3dymax, d3dny
            if (d3dnx*d3dny .gt. min(mvert,D3DMPT)) then
               call error ('domisc', 'Too many points on D3D file',
     &              warning)
               return
            endif
            d3dxstep = (d3dxmax - d3dxmin) / (d3dnx - 1)
            d3dystep = (d3dymax - d3dymin) / (d3dny - 1)
            read (lud3d,*) ((d3dz(i+(j-1)*d3dny), i = 1, d3dnx),
     &                     j = 1, d3dny)
            call fclose (lud3d)
c
c...........Search for special options on the READD3D order:
c
            color_mode = color_none
            zuse = zuse_height
            d3dzclip = .false.
            cookfactor = 1d0
            cook_method = cook_linear
            word = getword (word, line, lp)
            lw = leng(word)
            word = uppcase (word)
            done1 = lw .le. 0
            do while (.not. done1)
               if (word(1:lw) .eq. 'COLORMODE') then
c
c.................Read the color scheme:
c
                  word = getword (word, line, lp)
                  write (uout,830) 'COLORMODE', word(1:lw)
                  word = uppcase (word)
                  lw = leng (word)
                  ok = .true.
                  if (word(1:lw) .eq. 'NONE') then
                     color_mode = color_none
                  else if (word(1:lw) .eq. 'BINARY') then
                     color_mode = color_binary
                  else if (word(1:lw) .eq. 'GREY') then
                     color_mode = color_grey
                  else if (word(1:lw) .eq. 'FULL') then
                     color_mode = color_full
                  else
                     ok = .false.
                  endif
               else if (word(1:lw) .eq. 'COLORSCALE') then
c
c.................Read the color mapping:
c
                  word = getword (word, line, lp)
                  write (uout,830) 'COLORSCALE', word(1:lw)
                  word = uppcase (word)
                  lw = leng (word)
                  ok = .true.
                  if (word(1:lw) .eq. 'LINEAR') then
                     cookfactor = 1d0
                     cook_method = cook_linear
                  else if (word(1:lw) .eq. 'LOG') then
                     cookfactor = 1d0
                     cook_method = cook_log
                  else if (word(1:lw) .eq. 'ATAN') then
                     cookfactor = 1d0
                     cook_method = cook_atan
                  else if (word(1:lw) .eq. 'TANH') then
                     cookfactor = 1d0
                     cook_method = cook_tanh
                  else if (word(1:lw) .eq. 'ATANF') then
                     cook_method = cook_atan
                     if (setdble (cookfactor, line, lp)) then
                        ok = .true.
                     else
                        cookfactor = 1d0
                     endif
                  else if (word(1:lw) .eq. 'TANHF') then
                     cook_method = cook_tanh
                     if (setdble (cookfactor, line, lp)) then
                        ok = .true.
                     else
                        cookfactor = 1d0
                     endif
                  endif
               else if (word(1:lw) .eq. 'ZUSE') then
c
c.................Read the usage of the z coordinate:
c
                  word = getword (word, line, lp)
                  write (uout,830) 'ZUSE', word(1:lw)
                  word = uppcase (word)
                  lw = leng (word)
                  ok = .true.
                  if (word(1:lw) .eq. 'HEIGHT') then
                     zuse = zuse_height
                  else if (word(1:lw) .eq. 'COLOR') then
                     zuse = zuse_color
                  else if (word(1:lw) .eq. 'BOTH') then
                     zuse = zuse_both
                  else
                     ok = .false.
                  endif
               else if (word(1:lw) .eq. 'ZCLIP') then
c
c.................Read the usage of the z coordinate:
c
                  ok = setdble (d3dzmin, line, lp)
                  ok = ok .and. setdble (d3dzmax, line, lp)
                  if (ok) then
                     d3dzclip = .true.
                     write (uout,840) 'ZCLIP range', d3dzmin, d3dzmax
                  else
                     call error ('domisc', 'Error in zclip values',
     &                    warning)
                  endif
               endif
               word = getword (word, line, lp)
               lw = leng(word)
               word = uppcase (word)
               done1 = lw .le. 0
            enddo
            if (color_mode .ne. color_none .and.
     &                zuse .eq. zuse_height)     zuse = zuse_both
            if (color_mode .eq. color_none .and.
     &                zuse .eq. zuse_color)      zuse = zuse_both
c
c...........Store the surface points:
c
*
            nvert = 0
*
            fmin = 1e30
            fmax = -1e30
            ipt0 = nvert
            do j = 1, d3dny
               j1 = (j-1) * d3dny
               yy = d3dymin + (j-1) * d3dystep
               do i = 1, d3dnx
                  i1 = j1 + i
                  xx = d3dxmin + (i-1) * d3dxstep
                  nvert = nvert + 1
                  xvert(nvert) = xx
                  yvert(nvert) = yy
                  if (d3dzclip) then
                     if (d3dz(i1) .lt. d3dzmin) d3dz(i1) = d3dzmin
                     if (d3dz(i1) .gt. d3dzmax) d3dz(i1) = d3dzmax
                  endif
                  if (zuse .eq. zuse_color) then
                     zvert(nvert) = 0d0
                     fun_value(nvert) = d3dz(i1)
                  else
                     zvert(nvert) = d3dz(i1)
                     if (zuse.eq.zuse_both) fun_value(nvert) = d3dz(i1)
                  endif
                  if (d3dz(i1) .lt. fmin) fmin = d3dz(i1)
                  if (d3dz(i1) .gt. fmax) fmax = d3dz(i1)
               enddo
            enddo
            write (uout,812) fmin, fmax
c
c...........Transfrom the function values according to the color scale:
c
            if ( (zuse.eq.zuse_color .or. zuse.eq.zuse_both)
     &           .and. cook_method.ne.cook_linear) then
               if (cook_method .eq. cook_atan) then
                  do i = ipt0 + 1, nvert
                     fun_value(i) = atan(cookfactor * fun_value(i))
                  enddo
                  write (uout,814) 'ATAN', cookfactor
               else if (cook_method .eq. cook_tanh) then
                  do i = ipt0 + 1, nvert
                     fun_value(i) = tanh(cookfactor * fun_value(i))
                  enddo
                  write (uout,814) 'TANH', cookfactor
               else if (cook_method .eq. cook_log) then
c
c.................Check that a logarithm transform is correct:
c
                  ok = .true.
                  do i = ipt0 + 1, nvert
                     ok = ok .and. (fun_value(i) .gt. 0d0)
                  enddo
                  if (ok) then
                     do i = ipt0 + 1, nvert
                        fun_value(i) = log(fun_value(i))
                     enddo
                     write (uout,814) 'LOG', cookfactor
                  else
                     call error ('domisc',
     &                 'log colorscale impossible on negative D3D data',
     &                 warning)
                  endif
               endif
            endif
c
c...........Store the faces:
c
*
            nvf = 4
            isw1 = 0
            isw0 = 1
            nface(isw1) = 0
*
            do j = 1, d3dny-1
               j1 = (j-1) * d3dny
               j2 = j * d3dny
               do i = 1, d3dnx-1
                  i1j1 = ipt0 + j1 + i
                  i2j1 = ipt0 + j1 + i + 1
                  i1j2 = ipt0 + j2 + i
                  i2j2 = ipt0 + j2 + i + 1
                  nface(isw1) = nface(isw1) + 1
                  iface(1,nface(isw1),isw1) = i1j1
                  iface(2,nface(isw1),isw1) = i2j1
                  iface(3,nface(isw1),isw1) = i2j2
                  iface(4,nface(isw1),isw1) = i1j2
               enddo
            enddo
            write (uout,815) nvert, nface(isw1)
c
c.......................................................................
         else if (word(1:lw) .eq. 'READ3DLINES') then
c.......................................................................
c
c...........read in the name of the 3DLINES data file:
c
            file = getword (file, line, lp)
            lf = leng(file)
            write (uout,850) file(1:lf)
c
c...........Search for special options:
c
*
            defrgb(1) = 1.0
            defrgb(1) = 0.0
            defrgb(1) = 0.0
            defrgb(1) = 0.0
            deflinerad = 0.1d0
            defpointrad = 0.1d0
*
            con_mode = con_dots
            word = getword (word, line, lp)
            lw = leng(word)
            word = uppcase (word)
            done1 = lw .le. 0
            do while (.not. done1)
               if (word(1:lw) .eq. 'CONNECT') then
                  word = getword (word, line, lp)
                  write (uout,830) 'CONNECT', word(1:lw)
                  word = uppcase (word)
                  lw = leng (word)
                  ok = .true.
                  if (word(1:lw) .eq. 'DOTS') then
                     con_mode = con_dots
                  else if (word(1:lw) .eq. 'CYLINDERS') then
                     con_mode = con_cylinders
                  else if (word(1:lw) .eq. 'BOTH') then
                     con_mode = con_both
                  else
                     ok = .false.
                  endif
               else if (word(1:lw) .eq. 'DEFRGBF') then
                  if (setdble (xx, line, lp) .and.
     &                setdble (yy, line, lp) .and.
     &                setdble (zz, line, lp) .and.
     &                setdble (tt, line, lp)      ) then
                     defrgb(1) = xx
                     defrgb(2) = yy
                     defrgb(3) = zz
                     defrgb(4) = tt
                  endif
               else if (word(1:lw) .eq. 'DEFLINERAD') then
                  if (setdble (xx, line, lp)) then
                     deflinerad = xx
                  endif
               else if (word(1:lw) .eq. 'DEFPOINTRAD') then
                  if (setdble (xx, line, lp)) then
                     defpointrad = xx
                  endif
               endif
               word = getword (word, line, lp)
               lw = leng(word)
               word = uppcase (word)
               done1 = lw .le. 0
            enddo
c
c...........read in 3DLINES data file:
c
            if (fopen (lu3dl, file, ioread) .eq. ioerror) then
               call error ('domisc', 'I/O error opening 3D lines file',
     &              warning)
               return
            endif
            call read3dlines (lu3dl, con_mode, defrgb, deflinerad,
     &                        defpointrad, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'POVRAY') then
c.......................................................................
c
c...........Select output of a povray file:
c
            dopov = .true.
            if (leng(line) .gt. lp) filepov = getword (word, line, lp)
c
c.......................................................................
         else if (word(1:lw) .eq. 'OFF') then
c.......................................................................
c
c...........Select output of an OFF file:
c
            dooff = .true.
            if (leng(line) .gt. lp) fileoff = getword (word, line, lp)
c
c.......................................................................
         else if (word(1:lw) .eq. 'VRML') then
c.......................................................................
c
c...........Select output of an VRML file:
c
            dovrml = .true.
            if (leng(line) .gt. lp) filevrml = getword (word, line, lp)
c
c.......................................................................
         else if (word(1:lw) .eq. 'SET') then
c.......................................................................
c
            call setvariables (line, lp, uout)
c
c.......................................................................
         else if (word(1:1) .eq. '#') then
c.......................................................................
c
            continue
c
c.......................................................................
         else if (word(1:lw) .eq. 'ENDMISC') then
c.......................................................................
c
            done = .true.
c
c.......................................................................
         else
c.......................................................................
            call error ('parametric' , 'order not recognized', faterr)
            done = .true.
c
         endif
      enddo
c
      if (.not. (dooff .or. dopov .or. dovrml)) dooff = .true.
c
c
c.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.
c.+.+.+.+.+.+    OUTPUT THE RESULTS                           +.+.+.+.+.
c.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.
c
      if (dovrml) then
         line = 'Plot of a parametric surface'
         call writevrml1 (filevrml, line, uout)
      endif
      if (dooff) then
         line = 'Plot of a parametric surface'
         call writeoff (fileoff, line, uout)
      endif
      if (dopov) then
         call getnormals (ndeg, uout)
         if (ndeg .gt. 0) then
            call removedeg (ndeg, uout)
            call getnormals (ndeg, uout)
         endif
         line = 'Plot of a parametric surface'
         call writepov (filepov, line, uout)
      endif
c
c.....FORMATS:
c
 800  format (/
     & 1x, 'MISC: Miscellaneous graphical tasks.'/)
 810  format (/
     & 1x, 'MISC: Read in D3D file: ', a)
 812  format (
     & 1x, 'Min & max values of function (clipped): ', 1p, 2e14.6)
 814  format (
     & 1x, 'Colorscale mode:   ', a/
     & 1x, 'Colorscale factor: ', f12.4)
 815  format (
     & 1x, 'Created surface with ', i5, ' vertices and ', i5,
     &     ' quad faces')
 830  format (1x, a, 1x, a)
 840  format (1x, a, 1p, 2e14.6)
 850  format (/
     & 1x, 'MISC: Read in 3DLINES file: ', a)
c
      end
