c
c-----------------------------------------------------------------------
c
      logical function expression (exprvalue, nex, depend, ierr)
c
c.....expression - evaluate a precompiled expression.
c
c..Input parameters:
c    nex .......... number of the precompiled expression to be evaluated.
c
c..Output parameters:
c    expression ... true if the expression can be evaluated correctly,
c                   and false otherwise.
c    exprvalue .... value of the expression.
c    depend ....... true if the expression depends on an user variable
c                   and false otherwise.
c    ierr ......... error code (0 = no error).
c
c GLOBAL DATA:
c
      include           'implicit.inc'
      include           'strtools.inc'
      include           'error.inc'
      include           'expr.inc'
      include           'hash.inc'
      include           'opstack.inc'
      include           'tokens.inc'
c
c PARAMETERS:
c
      real*8            exprvalue
      integer           nex, ierr
      logical           depend
c
c LOCAL VARIABLES:
c
      integer           i, nvar, prec, sprec
      logical           done
      character*(80)    dummy
c
c.....initialize numbers and operators stacks:
c
      ierr = 0
      nnumstk = 0
      nopstk = 0
c
      depend = .false.
c
c.....loop over all tokens in the input expression:
c
      do i = exprfirsttk(nex), exprlasttk(nex)
         if (ierr .ne. 0) then
            expression = .false.
            ierr = 10 + ierr
            return
         endif
c
c.......................................................................
c
         if (tktype(i) .eq. tpvariable) then
c
c...........get user variable and store the value onto operands stack:
c
            depend = .true.
            nvar = tkoperator(i)
            nnumstk = nnumstk + 1
c
c...........TRICK: We will violate the encapsulation of the hash table
c           structure by directly recovering the value stored for the
c           nvar-th variable.
c
            numstk(nnumstk) = varrvalue(nvar)
c
c.......................................................................
c
         else if (tktype(i) .eq. tpnumber) then
c
c...........store number onto operands stack:
c
            nnumstk = nnumstk + 1
            numstk(nnumstk) = tknumval(i)
c
c.......................................................................
c
         else if (tktype(i) .eq. tpfunction) then
c
c...........store function onto operator stack:
c
            nopstk = nopstk + 1
            opstk(nopstk) = tkoperator(i)
            tyopstk(nopstk) = tpfunction
            precopstk(nopstk) = 0
***c
***c.......................................................................
***c
***         else if (tktype(i) .eq. tpunary) then
***c
***c...........store unary sign onto operator stack:
***c
***            nopstk = nopstk + 1
***            opstk(nopstk) = tkoperator(i)
***            tyopstk(nopstk) = tpunary
***            precopstk(nopstk) = 0
c
c.......................................................................
c
         else if (tktype(i) .eq. tpoperator .or.
     &            tktype(i) .eq. tpunary         ) then
c
c...........if precedence is larger than precedence of operator on
c           top of stack, pop and execute that one. In any case
c           push the operator just read in onto operator stack.
c...........To handle associativity, two precedence numbers will be
c           used: input and stack precedence. Input precedence will be
c           lower than stack precedence for the operators: +, -, *,
c           and /, in order to produce left associativity (i.e.
c           '4+3+2' will be interpreted as '(4+3)+2'). Contrarily,
c           input precedence of ^ will be higher than stack
c           precedence so that '4^3^2' will be interpreted as
c           '4^(3^2)'.
c
c             Precedence table:     | input  | stack  |
c           +-----------------------+--------+--------+
c           |       || (or)         |   1    |   2    |
c           |       && (and)        |   3    |   4    |
c           |  >  <  >=  <=  ==  != |   5    |   6    |
c           |    binary  +  -       |   7    |   8    |
c           |        *  /           |   9    |  10    |
c           |         ^             |  12    |  11    |
c           |    unary   +  -       |  13    |  14    |
c           +-----------------------+--------+--------+
c
            if (tktype(i) .eq. tpunary) then
               prec = 13
               sprec = 14
            else if (tkoperator(i).eq.ot_add .or.
     &               tkoperator(i).eq.ot_subs) then
               prec = 7
               sprec = 8
            else if (tkoperator(i).eq.ot_mult .or.
     &               tkoperator(i).eq.ot_div) then
               prec = 9
               sprec = 10
            else if (tkoperator(i) .eq. ot_elev) then
               prec = 12
               sprec = 11
            else if (tkoperator(i).eq.ot_gt  .or.
     &               tkoperator(i).eq.ot_lt  .or.
     &               tkoperator(i).eq.ot_ge  .or.
     &               tkoperator(i).eq.ot_le  .or.
     &               tkoperator(i).eq.ot_eq  .or.
     &               tkoperator(i).eq.ot_ne ) then
               prec = 5
               sprec = 6
            else if (tkoperator(i) .eq. ot_and) then
               prec = 3
               sprec = 4
            else if (tkoperator(i) .eq. ot_or) then
               prec = 1
               sprec = 2
            else
               call error ('expression',
     &              'Undefined operator', warning)
               ierr = 20
               expression = .false.
               return
            endif
            done = nopstk .le. 0
            do while (.not. done)
               if (prec .le. precopstk(nopstk)) then
                  call popandexecute (ierr)
                  if (ierr .ne. 0) then
                     expression = .false.
                     ierr = 40 + ierr
                     return
                  endif
                  done = nopstk .le. 0
               else
                  done = .true.
               endif
            enddo
            nopstk = nopstk + 1
            opstk(nopstk) = tkoperator(i)
            if (tktype(i) .eq. tpunary) then
               tyopstk(nopstk) = tpunary
            else
               tyopstk(nopstk) = tpoperator
            endif
            precopstk(nopstk) = sprec
c
c.......................................................................
c
         else if (tktype(i) .eq. tpopenpar) then
c
c...........push parenthesis onto the operator stack:
c
            nopstk = nopstk + 1
            opstk(nopstk) = tkoperator(i)
            tyopstk(nopstk) = tpopenpar
            precopstk(nopstk) = 0
c
c.......................................................................
c
         else if (tktype(i). eq. tpcomma) then
c
c...........pop and execute operators until an open parenthesis or
c           a comma is found. Push then comma onto op stack:
c
            if (nopstk .le. 0) then
               dummy='close parenthesis error'
               call error ('expression', dummy, warning)
               expression = .false.
               ierr = 50
               return
            endif
            do while (tyopstk(nopstk) .ne. tpopenpar .and.
     &                tyopstk(nopstk) .ne. tpcomma         )
               call popandexecute (ierr)
               if (ierr .ne. 0) then
                  expression = .false.
                  ierr = 40 + ierr
                  return
               endif
               if (nopstk .le. 0) then
                  dummy='comma out of order'
                  call error ('expression', dummy, warning)
                  expression = .false.
                  ierr = 50
                  return
               endif
            enddo
            nopstk = nopstk + 1
            opstk(nopstk)     = tkoperator(i)
            tyopstk(nopstk)   = tpcomma
            precopstk(nopstk) = 0
c
c.......................................................................
c
         else if (tktype(i) .eq. tpclosepar) then
c
c...........pop and execute operators until an open parenthesis is
c           found. If the operator on top is then a unary sign, pop
c           and execute it. If it is a function pop it and check
c           again for a unary sign:
c
            if (nopstk .le. 0) then
               dummy='close parenthesis error'
               call error ('expression', dummy, warning)
               expression = .false.
               ierr = 30
               return
            endif
            do while (tyopstk(nopstk) .ne. tpopenpar)
               call popandexecute (ierr)
               if (ierr .ne. 0) then
                  expression = .false.
                  ierr = 40 + ierr
                  return
               endif
               if (nopstk .le. 0) then
                  dummy='close parenthesis error'
                  call error ('expression', dummy, warning)
                  expression = .false.
                  ierr = 30
                  return
               endif
            enddo
            nopstk = nopstk - 1
            if (nopstk .ge. 1) then
               if (tyopstk(nopstk) .eq. tpunary) then
                  call popandexecute (ierr)
                  if (ierr .ne. 0) then
                     ierr = 40 + ierr
                     expression = .false.
                     return
                  endif
               endif
               if (tyopstk(nopstk) .eq. tpfunction) then
                  call popandexecute (ierr)
                  if (ierr .ne. 0) then
                     ierr = 40 + ierr
                     expression = .false.
                     return
                  endif
                  if (tyopstk(nopstk) .eq. tpunary) then
                     call popandexecute (ierr)
                     if (ierr .ne. 0) then
                        ierr = 40 + ierr
                        expression = .false.
                        return
                     endif
                  endif
               endif
            endif
c
c.......................................................................
c
         endif
c
      enddo
      if (ierr .ne. 0) then
         expression = .false.
         ierr = 10 + ierr
         return
      endif
c
c.....pop and execute the operators remaining on stack:
c
      do while (nopstk .gt. 0)
         call popandexecute (ierr)
         if (ierr .ne. 0) then
            ierr = 40 + ierr
            expression = .false.
            return
         endif
      enddo
c
c.....final result should be the single value stored in the numbers
c     stack:
c
      if (nnumstk .eq. 1) then
         expression = .true.
         exprvalue = numstk(nnumstk)
         exprval(nex) = exprvalue
      else
         dummy='one and only one value should'//
     &      ' remain on stack after all operators have been done'
         call error ('expression', dummy, warning)
         expression = .false.
         ierr = 60
         exprvalue = numstk(nnumstk)
         exprval(nex) = exprvalue
      endif
      return
      end
