c
c-----------------------------------------------------------------------
c
      logical function fnexttoken (rval, operator, type, line, lp, ierr)
c
c.....fnexttoken - gets and classifies next token in line(lp:)
c
c.....Input parameters:
c     type ....... last token type. Initialize to tpnone in the first
c                  call.
c     line ....... line containing possible tokens.
c     lp ......... pointer to the first position in line to be studied.
c
c.....Output parameters:
c     rval ....... value of the token when it is a number.
c     operator ... value of the token when it is an operator or
c                  hash entry in the case of a variable.
c     type ....... token type.
c     fnexttoken . .true. when a new token has been found,
c                  .false. otherwise.
c     ierr ....... 0 if everithing was right, not 0 otherwise.
c
      include           'strtools.inc'
      include           'error.inc'
      include           'tokens.inc'
c
      real*8            rval
      integer           operator, type, lp
      character*(*)     line
      character*(sstr)  dumbchar
c
      character*(lstr)  word, uppcase
      real*8            rvalue, oldrvalue
      logical           setdble, fgetvarname
      integer           oldtype, lp1, oldivalue, oldtp
      integer           h, hlookup
      logical           allowed(tpnone:tpvariable,tpnone:tpvariable)
      logical           T, F
      parameter         (T = .true., F = .false.)
      data ((allowed(i,j), j=tpnone,tpvariable), i=tpnone,tpvariable) /
     &         F, T, T, T, F, T, F, F, T,     !tpnone
     &         F, F, F, F, T, F, T, T, F,     !tpnumber
     &         F, F, F, F, F, T, F, F, F,     !tpfunction
     &         F, T, T, F, F, T, F, F, T,     !tpunary
     &         F, T, T, F, F, T, F, F, T,     !tpoperator
     &         F, T, T, T, F, T, F, T, T,     !tpopenpar
     &         F, T, T, T, F, T, F, F, T,     !tpcomma
     &         F, F, F, F, T, F, T, T, F,     !tpclosepar
     &         F, F, F, F, T, F, T, T, F  /   !tpvariable
c              !  !  !  !  !  !  !  !  !
c              !  !  !  !  !  !  !  !  tpvariable
c              !  !  !  !  !  !  !  tpclosepar
c              !  !  !  !  !  !  tpcomma
c              !  !  !  !  !  tpopenpar
c              !  !  !  !  tpoperator
c              !  !  !  tpunary
c              !  !  tpfunction
c              !  tpnumber
c              tpnone
c
c.....NOTE: allowed is used to determine wether a sucession of tokens
c     is correct or not. The adequeate use of this structure depends on
c     the fact that the token types are consecutive as follows:
c     tpnone, tpnumber, tpfunction, tpoperator, tpopenpar, tpcomma,
c     tpclosepar, tpvariable
c
c
c.....skip initial blanks:
c
      ierr = 0
      ll = leng(line)
      do while (line(lp:lp) .eq. blank)
         lp = lp + 1
      enddo
      if (lp .gt. ll) then
         fnexttoken = .false.
         return
      endif
c
c.....is token an open or close parenthesis?
c
      oldtype = type
      if (line(lp:lp) .eq. '(') then
         type = tpopenpar
         operator = ot_oppar
         lp = lp + 1
c
      else if (line(lp:lp) .eq. ')') then
         type = tpclosepar
         operator = ot_clospar
         lp = lp + 1
c
c.....is token a comma?
c
      else if (line(lp:lp) .eq. ',') then
         type = tpcomma
         operator = ot_comma
         lp = lp + 1
c
c.....is token an arithmetic operator?
c
      else if (line(lp:lp) .eq. '+') then
         if (oldtype.eq.tpnumber .or. oldtype.eq.tpclosepar
     &       .or. oldtype.eq.tpvariable) then
c
c...........an arithmetic + is legal at this position:
c
            type = tpoperator
            operator = ot_add
            lp = lp + 1
         else
c
c...........the + may belong to a real constant:
c
            lp1 = lp + 1
            if (setdble(rvalue, line, lp1)) then
               type = tpnumber
               rval = rvalue
               lp = lp1
            else
c
c..............let us suposse it is a unary sign:
c
               type = tpunary
               operator = ot_uplus
               lp = lp + 1
            endif
         endif
c
      else if (line(lp:lp) .eq. '-') then
         if (oldtype.eq.tpnumber .or. oldtype.eq.tpclosepar
     &       .or. oldtype.eq.tpvariable) then
c
c...........an arithmetic - is legal at this position:
c
            type = tpoperator
            operator = ot_subs
            lp = lp + 1
         else
c
c...........the - may belong to a real constant:
c
            lp1 = lp + 1
            if (setdble(rvalue, line, lp1)) then
               type = tpnumber
               rval = -rvalue
               lp = lp1
            else
c
c..............let us suposse it is a unary sign:
c
               type = tpunary
               operator = ot_uminus
               lp = lp + 1
            endif
         endif
c
      else if (line(lp:lp) .eq. '*') then
         type = tpoperator
         if (line(lp:lp+1) .eq. '**') then
            operator = ot_elev
            lp = lp + 1
         else
            operator = ot_mult
         endif
         lp = lp + 1
c
      else if (line(lp:lp) .eq. '/') then
         type = tpoperator
         operator = ot_div
         lp = lp + 1
c
      else if (line(lp:lp) .eq. '^') then
         type = tpoperator
         operator = ot_elev
         lp = lp + 1
c
      else if (line(lp:lp) .eq. '>') then
         type = tpoperator
         if (line(lp:lp+1) .eq. '>=') then
            operator = ot_ge
            lp = lp + 1
         else
            operator = ot_gt
         endif
         lp = lp + 1
c
      else if (line(lp:lp) .eq. '<') then
         type = tpoperator
         if (line(lp:lp+1) .eq. '<=') then
            operator = ot_le
            lp = lp + 1
         else
            operator = ot_lt
         endif
         lp = lp + 1
c
      else if (line(lp:lp+1) .eq. '==') then
         type = tpoperator
         operator = ot_eq
         lp = lp + 2
c
      else if (line(lp:lp+1) .eq. '!=') then
         type = tpoperator
         operator = ot_ne
         lp = lp + 2
c
      else if (line(lp:lp+1) .eq. '&&') then
         type = tpoperator
         operator = ot_and
         lp = lp + 2
c
      else if (line(lp:lp+1) .eq. '||') then
         type = tpoperator
         operator = ot_or
         lp = lp + 2
c
      else
c
c........is token a real value?
c
         lp1 = lp
         if (setdble (rvalue,line,lp1)) then
            type = tpnumber
            rval = rvalue
            lp = lp1
c
         else
c
c...........discarded other possibilities, it must be a function name,
c           a constant or a symbolic variable. All of them should
c           appear in the hash table:
c
            if (fgetvarname (word, line, lp)) then
               word = uppcase (word)
               h = hlookup (word, oldrvalue, oldivalue, oldtp)
               if (h .le. 0) then
                  dumbchar='Token not recognized: '//
     &                 word(1:leng(word))//newline//
     &                 ' Variables must be defined before used.'
                  call error ('fnexttoken', dumbchar,warning)
                  ierr = 2
                  fnexttoken = .false.
                  return
               else if (oldtp .eq. htfunction) then
                  type = tpfunction
                  operator = oldivalue
               else if (oldtp .eq. htconst) then
                  type = tpnumber
                  rval = oldrvalue
               else if (oldtp .eq. htvariable) then
                  type = tpvariable
                  rval = oldrvalue
                  operator = h
               endif
            else
               dumbchar='Error determining next token in:'//
     &             line(lp:ll)//
     &              newline//' Complete line follows: '//line(1:ll)
                  call error ('fnexttoken', dumbchar,warning)
               ierr = 3
               fnexttoken = .false.
               return
            endif
c
         endif
c
      endif
c
c.....is a type token allowed after an oldtype one?
c
      if (allowed (oldtype,type)) then
         fnexttoken = .true.
      else
         dumbchar=
     &      'this sequence of tokens is not allowed in this grammar'//
     &      newline//' offending sentence:'//line(1:ll)
         call error ('fnexttoken', dumbchar,warning)
         ierr = 1
         fnexttoken = .false.
      endif
      return
      end
