c
c-----------------------------------------------------------------------
c
      real*8 function gradient (var, grad, nv2, sumgrad, energ)
c
c.....gradient - compute the gradient vector of the electrostatic energy
c     of a collection of +1 point charges on the surface of a sphere of
c     readius 1. Theindependent variables are the (theta,phi) angular
c     coordinates of the N particles.
c
      include           'implicit.inc'
      include           'polyhedron.inc'
c
      integer           nv2
      real*8            var(nv2), grad(nv2), sumgrad, energ
c
      integer           i, j, ii
      real*8            sp, cp
      real*8            sumx(mvert), sumy(mvert), sumz(mvert)
      real*8            xij, yij, zij, rij, rij2, rij3
      real*8            sth(mvert), cthcph(mvert), cthsph(mvert)
c
c.....Determine angular coordinates of the starting points:
c
      do i = 1, nvert
         ii = nvert + i
         sth(i) = sin(var(i))
         cth(i) = cos(var(i))
         cp = cos(var(ii))
         sp = sin(var(ii))
         sthcph(i) = sth(i) * cp
         sthsph(i) = sth(i) * sp
         cthcph(i) = cth(i) * cp
         cthsph(i) = cth(i) * sp
         sumx(i) = 0d0
         sumy(i) = 0d0
         sumz(i) = 0d0
      enddo
c
      energ = 0d0
      do i = 1, nvert
         do j = 1, i-1
            xij = sthcph(i) - sthcph(j)
            yij = sthsph(i) - sthsph(j)
            zij = cth(i) - cth(j)
            rij2 = 1d0 / (xij*xij + yij*yij + zij*zij)
            rij = sqrt(rij2)
            rij3 = rij * rij2
            energ = energ + rij
            sumx(i) = sumx(i) + xij * rij3
            sumy(i) = sumy(i) + yij * rij3
            sumz(i) = sumz(i) + zij * rij3
            sumx(j) = sumx(j) - xij * rij3
            sumy(j) = sumy(j) - yij * rij3
            sumz(j) = sumz(j) - zij * rij3
         enddo
      enddo
c
      sumgrad = 0d0
      do i = 1, nvert
         ii = nvert + i
         grad(i) = -0.5d0 * (cthcph(i)*sumx(i) + cthsph(i)*sumy(i) -
     &                       sth(i)*sumz(i))
         grad(ii) = 0.5d0 * (sthsph(i)*sumx(i) - sthcph(i)*sumy(i))
         sumgrad = sumgrad + grad(i)*grad(i) + grad(ii)*grad(ii)
      enddo
      gradient = sumgrad
c
      return
      end
