c
c-----------------------------------------------------------------------
c
      subroutine hsv2rgb (h, s, v, r, g, b)
c
c.....hsv2rgb - Converts HSV color coordinates to RGB ones.
c     After Foley, et al, Computer Graphics ().
c
c H (=Hue) is interpreted in the range [0,360] degrees, and it
c          *must* be positive.
c          H:{0,60,120,180,240,300} are interpreted as {red, yellow,
c          green, cyan, blue, magenta}.
c S (=Saturation) and V (=value or brightness) *must* be in [0,1].
c R (=red), G (=green), and B (=blue) are reals in [0,1].
c
      real*8            h, s, v, r, g, b
      real*8            h1, f, p, q, t
      integer           i
c
      if (s .eq. 0.0) then
c
c........Hue is ignored, and v alone determines the whiteness:
c
         r = v
         g = v
         b = v
c
      else
         h1 = (mod (int(h),360)) / 60.0
*mdc*if floor
*         i = floor (h1)
*mdc*else
         if (h1. ge. 0.0) then
            i = int(h1)
         else if (h1-int(h1) .eq. 0.0) then
            i = int(h1)
         else
            i = int(h1) - 1
         endif
*mdc*endif
         f = h1 - i
         p = v * (1 - s)
         q = v * (1 - s*f)
         t = v * (1 - s * (1-f))
         if (i .eq. 0) then
            r = v
            g = t
            b = p
         else if (i .eq. 1) then
            r = q
            g = v
            b = p
         else if (i .eq. 2) then
            r = p
            g = v
            b = t
         else if (i .eq. 3) then
            r = p
            g = q
            b = v
         else if (i .eq. 4) then
            r = t
            g = p
            b = v
         else if (i .eq. 5) then
            r = v
            g = p
            b = q
         endif
      endif
      return
      end
