c
c-----------------------------------------------------------------------
c
      subroutine molws (
     &  xct, yct, zct,     !Polyhedron center
     &  usedefrgb,         !Use default colors?
     &  rgb,               !Use this color instead
     &  itexture,          !POV texture index
     &  dofaces,           !Plot the ployhedra as faces
     &  doedges,           !Plot the face edges
     &  doirreducible,     !Plot the irreducible part of the WS cell
     &  stickrad,          !Edges radii
     &  uout)
c
c.....molws - Add a frame representing the Wigner-Seitz cell.
c             Routine created by Angel Martin Pendas for the pairpot3
c             code (c) 1995-97.
c             Adapted for tessel2 by VLC in 1997.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'molge.inc'
      include           'spgcom.inc'
c
      integer           uout, itexture
      real*8            xct, yct, zct, stickrad, rgb(4)
      logical           usedefrgb, dofaces, doedges, doirreducible
c
      real*8            EPSDIS
      parameter         (EPSDIS = 1d-6)
      integer           MAXNN
      parameter         (MAXNN = 26)
      integer           nn, ix(MAXNN), iy(MAXNN), iz(MAXNN)
      real*8            xn(3,MAXNN), rk2(26)
      logical           con(MAXNN,MAXNN)
      integer           i, j, k, l, n, nco1, nco2, nco3, idet, ncount
      integer           ii, ij, jj, ik, jk, kk, npt, ip1, ip2, ir1, ir2
      integer           i0, i1, i2, i3, i4, j1, j2, j3, j4, j5, j6, k1
      integer           nequ, ipol, nsid
      integer           oldplh, oldpt, oldpol, oldstk
      real*8            xnor2, xnorm2, detd, cx, cy, cz, pesca
      real*8            cxx, cyy, czz, hh, pmatrix(4)
      real*8            x1, y1, z1, x2, y2, z2, x3, y3, z3, h1, h2, h3
      real*8            xpr1, ypr1, zpr1, xpr2, ypr2, zpr2
      real*8            xr1, yr1, zr1, xr2, yr2, zr2, xmed, ymed, zmed
      logical           isnew, discard, doagain, reduce
c
      integer           MPOLIG, MSIDES
      parameter         (MPOLIG = 400, MSIDES = GMSIDE)
      integer           npolig, nsides(MPOLIG), iside(MSIDES,MPOLIG)
      real*8            normal(4,MPOLIG)
      logical           active(MPOLIG)
c
      integer           MPOLH
      parameter         (MPOLH = 20)
      integer           ntetrag, nirrface, irrface(MPOLIG)
      real*8            tetrag(3,3,MPOLH)
c
      logical           polyjoin
c
      write (uout,800)
c
      oldpt = gnpt
      oldplh = gnplh
      oldpol = gnpol
      oldstk = gnstk
c
*mdc*debug
*      write (uout,6661) 'DEBUG(molws): '
*      write (uout,6661) 'ared.... ', ared, bred, cred
*      write (uout,6661) 'arec.... ', arec, brec, crec
*      write (uout,6661) 'cosdir.. ', calph, cbeta, cgamm
*      write (uout,6661) 'g....... ', (g(i),i=1,6)
*      write (uout,6661) 'gr...... ', (gr(i),i=1,6)
*      write (uout,6661) 'gp...... ', ((gp(i,j),j=1,i),i=1,3)
*      write (uout,6661) 'grp..... ', ((grp(i,j),j=1,i),i=1,3)
*      write (uout,6661) 'lcent... ', lcent
*      write (uout,6661) 'tocrec.. ', ((tocrec(lcent,i,j),j=1,3),i=1,3)
* 6661 format (1x, a, 20f12.6)
*mdc*endif
c
c.....Get the list of points that may contribute to the voronoi
c     polyhedron:
c
      xnor2 = max (ared, bred, cred)  ! to normalize distances
      xnor2 = xnor2 * xnor2
      xnorm2 = 0.5d0 / xnor2
      n = 0
      do i = -1, 1
         ii = i * i
         do j = -1, 1
            ij = i * j
            jj = j * j
            do k = -1, 1
               if (i.ne.0 .or. j.ne.0 .or. k.ne.0) then
                  ik = i * k
                  jk = j * k
                  kk = k * k
                  n = n + 1
                  ix(n) = i
                  iy(n) = j
                  iz(n) = k
                  rk2(n) = gp(1,1)*ii + 2*gp(1,2)*ij + 2*gp(1,3)*ik +
     &                     gp(2,2)*jj + 2*gp(2,3)*jk + gp(3,3)*kk
                  rk2(n) = rk2(n) * xnorm2
               endif
            enddo
         enddo
      enddo
c
c.....Run over triplets of points. Find the center of the spheres made
c     up by the origin and the triplet under consideration.
c     (This is an implementation of the Finney algorithm, known to scale
c     as N**3 much less efficient than the N*log(N) scaling of the Quick
c     Hull algorithm. Finney's method is, however, much simpler and the
c     convex hull we are looking for has very few vertices).
c     (Ref.: J. Comput. Phys. 32 (1979) 137)
c
      nn = 0
      do i = 3, 26
         do j = 2, i-1
            do k = 1, j-1
               nco1 =  iy(j)*iz(k) - iz(j)*iy(k)
               nco2 = -iy(i)*iz(k) + iz(i)*iy(k)
               nco3 =  iy(i)*iz(j) - iz(i)*iy(j)
               idet = ix(i)*nco1 + ix(j)*nco2 + ix(k)*nco3
               if (idet.ne.0) then
                  detd = rk2(i)*nco1 + rk2(j)*nco2 + rk2(k)*nco3
                  cx = detd / idet
                  nco1 = -ix(j)*iz(k) + iz(j)*ix(k)
                  nco2 =  ix(i)*iz(k) - iz(i)*ix(k)
                  nco3 = -ix(i)*iz(j) + iz(i)*ix(j)
                  detd = rk2(i)*nco1 + rk2(j)*nco2 + rk2(k)*nco3
                  cy = detd / idet
                  nco1 =  ix(j)*iy(k) - iy(j)*ix(k)
                  nco2 = -ix(i)*iy(k) + iy(i)*ix(k)
                  nco3 =  ix(i)*iy(j) - iy(i)*ix(j)
                  detd = rk2(i)*nco1 + rk2(j)*nco2 + rk2(k)*nco3
                  cz = detd / idet
c
c.................Is this a vertex of the Voronoi hull?
c
                  ncount = 0
                  do l = 1, 26
                     pesca = ix(l)*cx + iy(l)*cy + iz(l)*cz
                     if (pesca .gt. rk2(l)) goto 111
                     ncount = ncount + 1
                  enddo
 111              if (ncount.eq.26) then
c
c....................A vertex has been found. Check if it is really new:
c
                     isnew = .true.
                     l = 1
                     do while (isnew .and. l.le.nn)
                        isnew = abs(cx-xn(1,l)) .gt. EPSDIS .or.
     &                          abs(cy-xn(2,l)) .gt. EPSDIS .or.
     &                          abs(cz-xn(3,l)) .gt. EPSDIS
                        l = l + 1
                     enddo
                     if (isnew) then
                        nn = nn + 1
                        xn(1,nn) = cx
                        xn(2,nn) = cy
                        xn(3,nn) = cz
                     endif
                  endif
               endif
            enddo
         enddo
      enddo
c
c.....Store the vertices in centered coordinates:
c
      do i = 1, nn
         cxx = tocrec(lcent,1,1) * xn(1,i) + 
     &         tocrec(lcent,2,1) * xn(2,i) + 
     &         tocrec(lcent,3,1) * xn(3,i)
         cyy = tocrec(lcent,1,2) * xn(1,i) + 
     &         tocrec(lcent,2,2) * xn(2,i) + 
     &         tocrec(lcent,3,2) * xn(3,i)
         czz = tocrec(lcent,1,3) * xn(1,i) + 
     &         tocrec(lcent,2,3) * xn(2,i) + 
     &         tocrec(lcent,3,3) * xn(3,i)
         xn(1,i) = gr(1)*cxx + gr(2)*cyy + gr(4)*czz
         xn(2,i) = gr(2)*cxx + gr(3)*cyy + gr(5)*czz
         xn(3,i) = gr(4)*cxx + gr(5)*cyy + gr(6)*czz
c
c........Unrenormalize:
c
         xn(1,i) = xn(1,i) * xnor2
         xn(2,i) = xn(2,i) * xnor2
         xn(3,i) = xn(3,i) * xnor2
      enddo
c
c.....Build up the polyhedron data structure.
c
c.....Run over all non-null triangles among the selected points:
c
      npolig = 0
      do i = 1, nn
         x1 = xn(1,i)
         y1 = xn(2,i)
         z1 = xn(3,i)
         do j = i+1, nn
            x2 = xn(1,j)
            y2 = xn(2,j)
            z2 = xn(3,j)
            do k = j+1, nn
               x3 = xn(1,k)
               y3 = xn(2,k)
               z3 = xn(3,k)
               h1 = (y3-y1)*(x2-x1) - (x3-x1)*(y2-y1)
               h2 = (z3-z1)*(x2-x1) - (x3-x1)*(z2-z1)
               h3 = (z3-z1)*(y2-y1) - (y3-y1)*(z2-z1)
c
c..............discard polygons between colinear points:
c
               if (abs(h1)+abs(h2)+abs(h3) .gt. 1d-7) then
c
c.................get the Robert's representation of this polygon
c                 (the sign is defined by requiring that the origin
c                 is *inside* the convex polyhedron):
c                 Equation of the plane: ax+by+cz+d=0.
c                 (a,b,c,d)=(pmatrix)
c
                  pmatrix(1) = (y1-y2)*z3 + (y3-y1)*z2 + (y2-y3)*z1
                  pmatrix(2) = (x2-x1)*z3 + (x1-x3)*z2 + (x3-x2)*z1
                  pmatrix(3) = (x1-x2)*y3 + (x3-x1)*y2 + (x2-x3)*y1
                  pmatrix(4) = x1*(y2*z3-y3*z2) + y1*(x3*z2-x2*z3) +
     &                         z1*(x2*y3-x3*y2)
                  if (pmatrix(4).lt.0d0) then
                     pmatrix(1) = -pmatrix(1)
                     pmatrix(2) = -pmatrix(2)
                     pmatrix(3) = -pmatrix(3)
                     pmatrix(4) = -pmatrix(4)
                  endif
c
c.................if this triangle is an exterior face of the convex
c                 polyhedron, all the vertices must lie in the face
c                 or in the *positive* side of the plane:
c
                  discard = .false.
                  npt = 0
                  do while (.not.discard .and. npt.lt.nn)
                     npt = npt + 1
                     hh = xn(1,npt)*pmatrix(1) + xn(2,npt)*pmatrix(2) +
     &                    xn(3,npt)*pmatrix(3) + pmatrix(4)
                     discard = (hh .lt. -1d-12)
                  enddo
c
c.................store out all non discarded triangles:
c
                  if (.not.discard) then
                     npolig = npolig + 1
                     if (npolig.gt.MPOLIG) call error ('molws',
     &                   'npolig > MPOLIG', faterr)
                     nsides(npolig) = 3
                     iside(1,npolig) = i
                     iside(2,npolig) = j
                     iside(3,npolig) = k
                     normal(1,npolig) = pmatrix(1)
                     normal(2,npolig) = pmatrix(2)
                     normal(3,npolig) = pmatrix(3)
                     normal(4,npolig) = pmatrix(4)
                     active(npolig) = .true.
                  endif
               endif
            enddo
         enddo
      enddo
c
c.....Reduce coplanar polygons with two or more common vertices:
c
      doagain = .true.
      do while (doagain)
c
c........repeat cycles until no more reductions occur:
c
         doagain = .false.
         do i = 1, npolig
            do j = i+1, npolig
               if (active(i) .and. active(j)) then
                  reduce = polyjoin (i, j, npolig, nsides, iside,
     &                     MPOLIG, MSIDES, nn, xn, MAXNN)
                  if (reduce) then
                     doagain = .true.
                     active(j) = .false.
                  endif
               endif
            enddo
         enddo
      enddo
c
c.....Write out the vertices, polygons and polyhedra to the polyhedron
c     data structure.
c
      gnplh = gnplh + 1
      if (gnplh .gt. GMPLH) call error ('molws',
     &   'Too many polyhedra. Increase GMPLH!', faterr)
      gplhrgbf(1,gnplh) = rgb(1)
      gplhrgbf(2,gnplh) = rgb(2)
      gplhrgbf(3,gnplh) = rgb(3)
      gplhrgbf(4,gnplh) = rgb(4)
      gplhtext(gnplh) = itexture
c
c.....store the new vertices:
c
      if (gnpt+nn+1 .gt. GMPT) call error ('molws',
     &    'Too many vertices. Increase GMPT!', faterr)
      gnvert(gnplh) = nn
      do i = 1, nn
         gnpt = gnpt + 1
         gxpt(gnpt) = xn(1,i)
         gypt(gnpt) = xn(2,i)
         gzpt(gnpt) = xn(3,i)
         givert(i,gnplh) = gnpt
      enddo
      gnpt = gnpt + 1
      gxpt(gnpt) = xn(1,i)
      gypt(gnpt) = xn(2,i)
      gzpt(gnpt) = xn(3,i)
      gctplh(gnplh) = gnpt
c
c.....store the faces:
c
      gnface(gnplh) = 0
      do i = 1, npolig
         if (active(i)) then
            gnpol = gnpol + 1
            if (gnpol .gt. GMPOL) call error ('molws',
     &         'Too many polygons. Increase GMPOL!', faterr)
c
c...........sides of this face:
c
            gnside(gnpol) = nsides(i)
            if (gnside(gnpol) .gt. GMSIDE) call error ('molws',
     &         'Too many sides in a polygon. Increase GMSIDE!', faterr)
            do j = 1, nsides(i)
               giside(j,gnpol) = oldpt + iside(j,i)
            enddo
            gnormal(1,gnpol) = normal(1,i)
            gnormal(2,gnpol) = normal(2,i)
            gnormal(3,gnpol) = normal(3,i)
            gnormal(4,gnpol) = normal(4,i)
            gnface(gnplh) = gnface(gnplh) + 1
            if (gnface(gnplh).gt.GMFACE) call error ('molws',
     &            'Too many faces in a polyhedron. Increase GMFACE',
     &            faterr)
            giface(gnface(gnplh),gnplh) = gnpol
         endif
      enddo
c
      write (uout,805) gnpt-oldpt
      do i = 1, nn
         write (uout,810) i, xn(1,i), xn(2,i), xn(3,i)
      enddo
      write (uout,815) gnface(gnplh)
      i1 = 0
      do i = 1, npolig
         if (active(i)) then
            i1 = i1 + 1
            write (uout,820) i1, nsides(i),
     &                       (iside(j,i), j = 1, nsides(i))
         endif
      enddo
*mdc*if debug
*      write (uout,*) '*** Vertices (cartesian coord.):', nn
*      do i = oldpt + 1, gnpt
*         write (uout,6663) i, gxpt(i), gypt(i), gzpt(i)
*      enddo
*      write (uout,*) '*** Polygons:', npolig
*      write (uout,*) ' pol activ nside iside... (normals)'
*      do i = 1, npolig
*         if (active(i)) then
*            write (uout,*) active(i), nsides(i),
*     &                     (iside(j,i), j = 1, nsides(i)), '(',
*     &                     (normal(k,i), k = 1, 4), ')'
*         else
*            write (uout,*) active(i), nsides(i),
*     &                     (iside(j,i), j = 1, nsides(i))
*         endif
*      enddo
* 6663 format (1x, i3, 3f10.4)
*mdc*endif
c
c.......................................................................
      if (doirreducible) goto 1
c.......................................................................
c
      if (.not. dofaces) then
         gnplh = gnplh - 1
         gnpol = oldpol
      endif
c
      if (doedges) then
         do i = 1, n
            do j = 1, n
               con(i,j) = .false.
            enddo
         enddo
         do i = 1, npolig
            if (active(i)) then
               do j = 1, nsides(i)
                  j1 = j + 1
                  if (j1 .gt. nsides(i)) j1 = 1
                  con(iside(j,i),iside(j1,i)) = .true.
                  con(iside(j1,i),iside(j,i)) = .true.
               enddo
            endif
         enddo
         do i = 1, n
            do j = 1, i-1
               if (con(i,j)) then
                  gnstk = gnstk + 1
                  if (gnstk.gt.GMSTK) call error ('molws',
     &                'Number of sticks larger than GMSTK', faterr)
                  gstkpt(1,gnstk) = i + oldpt
                  gstkpt(2,gnstk) = j + oldpt
                  gstkrad(gnstk) = stickrad
                  gstkdefrgb(gnstk) = usedefrgb
                  gstkrgbf(1,gnstk) = rgb(1)
                  gstkrgbf(2,gnstk) = rgb(2)
                  gstkrgbf(3,gnstk) = rgb(3)
                  gstkrgbf(4,gnstk) = rgb(4)
                  gstktext(gnstk) = itexture
               endif
            enddo
         enddo
      endif
c
c.......................................................................
      if (.not. doirreducible) goto 2
c.......................................................................
c
c.......................................................................
 1    continue
c.......................................................................
c
c
c.....Determine the irreducible part of the Wigner-Seitz cell.
c     Substitute the WS cell just obtained.
c
      do i = 1, gnface(gnplh)
         active(i) = .true.
      enddo
c
c.....Apply the symmetry operations to the normals of each face and
c     get the list of faces different by symmetry. Only the directions
c     and not the orientation of the normal is significant:
c
      nirrface = 0
      do i = 1, gnface(gnplh)
         i1 = giface(i,gnplh)
         if (active(i)) then
            nirrface = nirrface + 1
            if (nirrface.gt.MPOLIG) call error ('molws',
     &         'nirrface > MPOLIG. Increase MPOLIG!', faterr)
            irrface(nirrface) = i1
c
c...........run over the symmetry operations:
c
            do j = 1, neqv
               h1 = gnormal(1,i1) * rotm(1,1,j) +
     &              gnormal(2,i1) * rotm(1,2,j) +
     &              gnormal(3,i1) * rotm(1,3,j)
               h2 = gnormal(1,i1) * rotm(2,1,j) +
     &              gnormal(2,i1) * rotm(2,2,j) +
     &              gnormal(3,i1) * rotm(2,3,j)
               h3 = gnormal(1,i1) * rotm(3,1,j) +
     &              gnormal(2,i1) * rotm(3,2,j) +
     &              gnormal(3,i1) * rotm(3,3,j)
c
c..............is this transformed face still active ?
c
               do k = 1, gnface(gnplh)
                  k1 = giface(k,gnplh)
                  if (active(k) .and. k.ne.i) then
                     if ( abs(h1-gnormal(1,k1)).lt.EPSDIS .and.
     &                    abs(h2-gnormal(2,k1)).lt.EPSDIS .and.
     &                    abs(h3-gnormal(3,k1)).lt.EPSDIS)
     &                  active(k) = .false.
c
c....................CAUTION: WS cell are centrosymmetric:
c
                     if ( abs(h1+gnormal(1,k1)).lt.EPSDIS .and.
     &                    abs(h2+gnormal(2,k1)).lt.EPSDIS .and.
     &                    abs(h3+gnormal(3,k1)).lt.EPSDIS)
     &                  active(k) = .false.
                  endif
               enddo
            enddo
         endif
      enddo
      write (uout,825) (irrface(i), i = 1, nirrface)
*mdc*if debug
*      write (uout,*) 'DEBUG(molws): Irreducible faces: ',nirrface
*      do i = 1, nirrface
*         i1 = irrface(i)
*         write (uout,*) i1, (gnormal(j,i1),j=1,3)
*      enddo
*mdc*endif
c
c.....Apply the symmetry operations to determine the independent edges:
c
      ntetrag = 0
      do i = 1, nirrface
         i1 = irrface(i)
         ipol = giface(i1,gnplh)
c
c........middle point in this face:
c
         xmed = 0d0
         ymed = 0d0
         zmed = 0d0
         nsid = gnside(ipol)
         do j = 1, nsid
            ip1 = giside(j,ipol)
            xmed = xmed + gxpt(ip1)
            ymed = ymed + gypt(ip1)
            zmed = zmed + gzpt(ip1)
         enddo
         xmed = xmed / nsid
         ymed = ymed / nsid
         zmed = zmed / nsid
c
c........initially declare all sides to be active:
c
         do j = 1, nsid
            active(j) = .true.
         enddo
c
c........apply the symmetry operations to declare inactive the segments
c        that are equivalent to previous segments:
c
         do j = 1, nsid
            if (active(j)) then
               ip1 = giside(j,ipol)
               ip2 = giside(mod(j,nsid)+1,ipol)
               x1 = gxpt(ip1)
               y1 = gypt(ip1)
               z1 = gzpt(ip1)
               x2 = gxpt(ip2)
               y2 = gypt(ip2)
               z2 = gzpt(ip2)
c
c..............apply the symmetry operations to this side:
c
               do k = 1, neqv
                  xpr1 = x1*rotm(1,1,k)+y1*rotm(1,2,k)+z1*rotm(1,3,k)
                  ypr1 = x1*rotm(2,1,k)+y1*rotm(2,2,k)+z1*rotm(2,3,k)
                  zpr1 = x1*rotm(3,1,k)+y1*rotm(3,2,k)+z1*rotm(3,3,k)
                  xpr2 = x2*rotm(1,1,k)+y2*rotm(1,2,k)+z2*rotm(1,3,k)
                  ypr2 = x2*rotm(2,1,k)+y2*rotm(2,2,k)+z2*rotm(2,3,k)
                  zpr2 = x2*rotm(3,1,k)+y2*rotm(3,2,k)+z2*rotm(3,3,k)
c
                  do l = 1, nsid
                     if (active(l)) then
c
                        ir1 = giside(l,ipol)
                        ir2 = giside(mod(l,nsid)+1,ipol)
                        xr1 = gxpt(ir1)
                        yr1 = gypt(ir1)
                        zr1 = gzpt(ir1)
                        xr2 = gxpt(ir2)
                        yr2 = gypt(ir2)
                        zr2 = gzpt(ir2)
c
c.......................Compare if sides different and sym op diff
c                       Note: A side is equivalent to itself IFF a
c                       symop other than *1* interchanges its vertices
c
                        nequ = 0
                        if ( abs(xpr1-xr1).lt.EPSDIS .and.
     &                       abs(ypr1-yr1).lt.EPSDIS .and.
     &                       abs(zpr1-zr1).lt.EPSDIS .and.
     &                       abs(xpr2-xr2).lt.EPSDIS .and.
     &                       abs(ypr2-yr2).lt.EPSDIS .and.
     &                       abs(zpr2-zr2).lt.EPSDIS ) then
                           nequ = 1
                        else if ( abs(xpr1-xr2).lt.EPSDIS .and.
     &                            abs(ypr1-yr2).lt.EPSDIS .and.
     &                            abs(zpr1-zr2).lt.EPSDIS .and.
     &                            abs(xpr2-xr1).lt.EPSDIS .and.
     &                            abs(ypr2-yr1).lt.EPSDIS .and.
     &                            abs(zpr2-zr1).lt.EPSDIS ) then
                           nequ = -1
                        endif
                        if (nequ.ne.0 .and.k.ne.1) then
                           if (l.eq.j) then
                              if (nequ.eq.-1) active(l)=.false.
                           else
                              active(l) = .false.
                           endif
                        endif
c
                        nequ = 0
                        if ( abs(xpr1+xr1).lt.EPSDIS .and.
     &                       abs(ypr1+yr1).lt.EPSDIS .and.
     &                       abs(zpr1+zr1).lt.EPSDIS .and.
     &                       abs(xpr2+xr2).lt.EPSDIS .and.
     &                       abs(ypr2+yr2).lt.EPSDIS .and.
     &                       abs(zpr2+zr2).lt.EPSDIS ) then
                           nequ = 1
                        else if ( abs(xpr1+xr2).lt.EPSDIS .and.
     &                            abs(ypr1+yr2).lt.EPSDIS .and.
     &                            abs(zpr1+zr2).lt.EPSDIS .and.
     &                            abs(xpr2+xr1).lt.EPSDIS .and.
     &                            abs(ypr2+yr1).lt.EPSDIS .and.
     &                            abs(zpr2+zr1).lt.EPSDIS ) then
                           nequ = -1
                        endif
                        if (nequ.ne.0) then
                           if (l.eq.j) then
                              if (nequ.eq.-1) active(l)=.false.
                           else
                              active(l) = .false.
                           endif
                        endif
                     endif
                  enddo
               enddo
c
c..............write out a new independent tetrahedron:
c
               if (.not. active(j)) then
c
c.................(a) the side is equivalent to itself. The vertices
c                     of the tetrahedron are:
c
c                 Implicit corner: the (0,0,0) point
c
                  ntetrag = ntetrag+1
                  if (ntetrag.gt.MPOLH) call error ('molws',
     &               'ntetrag > MPOLH. Increase MPOLH!', faterr)
c
c                 First corner: the middle point of the face
c
                  tetrag(1,1,ntetrag) = xmed
                  tetrag(1,2,ntetrag) = ymed
                  tetrag(1,3,ntetrag) = zmed
c
c                 Second corner: a vertex of the side
c
                  tetrag(2,1,ntetrag) = x1
                  tetrag(2,2,ntetrag) = y1
                  tetrag(2,3,ntetrag) = z1
c
c                 third corner: The middle point of the side
c
                  tetrag(3,1,ntetrag) = (x1+x2)*0.5d0
                  tetrag(3,2,ntetrag) = (y1+y2)*0.5d0
                  tetrag(3,3,ntetrag) = (z1+z2)*0.5d0
c
               else
c
c.................(b) this side, complete, is in the tetrahedron.
c                     The vertices of the tetrahedron are:
c
c                 Implicit corner: the (0,0,0) point
c
                  ntetrag = ntetrag+1
c
c                 First corner: the middle point of the face
c
                  tetrag(1,1,ntetrag) = xmed
                  tetrag(1,2,ntetrag) = ymed
                  tetrag(1,3,ntetrag) = zmed
c
c                 Second corner: a vertex of the side
c
                  tetrag(2,1,ntetrag) = x1
                  tetrag(2,2,ntetrag) = y1
                  tetrag(2,3,ntetrag) = z1
c
c                 Third corner: the other vertex of the side
c
                  tetrag(3,1,ntetrag) = x2
                  tetrag(3,2,ntetrag) = y2
                  tetrag(3,3,ntetrag) = z2
c
               endif
               active(j) = .false.
            endif
         enddo
      enddo
c
      write (uout,830) ntetrag
      do i = 1,ntetrag
         write (uout,835) i, (tetrag(1,j,i),j=1,3),
     &                    (tetrag(2,j,i),j=1,3), (tetrag(3,j,i),j=1,3)
      enddo
c
c.....write the irreducible WS zone to the graphic elements database:
c
      gnplh = oldplh
      gnpol = oldpol
      gnstk = oldstk
      gnpt = oldpt
c
      gnpt = gnpt + 1
      gxpt(gnpt) = 0d0
      gypt(gnpt) = 0d0
      gzpt(gnpt) = 0d0
      i0 = oldpt+1
      do i = 1, ntetrag
         i1 = gnpt+1
         i2 = gnpt+2
         i3 = gnpt+3
         i4 = gnpt+4
         gnpt = gnpt + 4
         if (gnpt .gt. GMPT) call error ('molws',
     &       'Too many vertices. Increase GMPT!', faterr)
         gxpt(i1) = tetrag(1,1,i)
         gypt(i1) = tetrag(1,2,i)
         gzpt(i1) = tetrag(1,3,i)
         gxpt(i2) = tetrag(2,1,i)
         gypt(i2) = tetrag(2,2,i)
         gzpt(i2) = tetrag(2,3,i)
         gxpt(i3) = tetrag(3,1,i)
         gypt(i3) = tetrag(3,2,i)
         gzpt(i3) = tetrag(3,3,i)
         gxpt(i4) = (gxpt(i0)+gxpt(i1)+gxpt(i2)+gxpt(i3)) * 0.25d0
         gypt(i4) = (gypt(i0)+gypt(i1)+gypt(i2)+gypt(i3)) * 0.25d0
         gzpt(i4) = (gzpt(i0)+gzpt(i1)+gzpt(i2)+gzpt(i3)) * 0.25d0
         if (dofaces) then
            gnplh = gnplh + 1
            if (gnplh .gt. GMPLH) call error ('molws',
     &         'Too many polyhedra. Increase GMPLH!', faterr)
            gnface(gnplh) = 4
            j1 = gnpol + 1
            j2 = gnpol + 2
            j3 = gnpol + 3
            j4 = gnpol + 4
            gnpol = gnpol + 4
            if (gnpol .gt. GMPOL) call error ('molws',
     &         'Too many polygons. Increase GMPOL!', faterr)
            giface(1,gnplh) = j1
            giface(2,gnplh) = j2
            giface(3,gnplh) = j3
            giface(4,gnplh) = j4
            gnvert(gnplh) = 4
            givert(1,gnplh) = i0
            givert(2,gnplh) = i1
            givert(3,gnplh) = i2
            givert(4,gnplh) = i3
            gctplh(gnplh) = i4
            gplhrgbf(1,gnplh) = rgb(1)
            gplhrgbf(2,gnplh) = rgb(2)
            gplhrgbf(3,gnplh) = rgb(3)
            gplhrgbf(4,gnplh) = rgb(4)
            gplhtext(gnplh) = itexture
            gnside(j1) = 3
            gnside(j2) = 3
            gnside(j3) = 3
            gnside(j4) = 3
            giside(1,j1) = 0
            giside(2,j1) = 1
            giside(3,j1) = 2
            giside(1,j2) = 0
            giside(2,j2) = 2
            giside(3,j2) = 3
            giside(1,j3) = 0
            giside(2,j3) = 1
            giside(3,j3) = 3
            giside(1,j4) = 1
            giside(2,j4) = 2
            giside(3,j4) = 3
         endif
         if (doedges) then
            j1 = gnstk + 1
            j2 = gnstk + 2
            j3 = gnstk + 3
            j4 = gnstk + 4
            j5 = gnstk + 5
            j6 = gnstk + 6
            gnstk = gnstk + 6
            if (gnstk.gt.GMSTK) call error ('molws',
     &         'Number of sticks larger than GMSTK', faterr)
            gstkpt(1,j1) = i0
            gstkpt(2,j1) = i1
            gstkpt(1,j2) = i0
            gstkpt(2,j2) = i2
            gstkpt(1,j3) = i0
            gstkpt(2,j3) = i3
            gstkpt(1,j4) = i1
            gstkpt(2,j4) = i2
            gstkpt(1,j5) = i2
            gstkpt(2,j5) = i3
            gstkpt(1,j6) = i3
            gstkpt(2,j6) = i1
            do j = j1, j6
               gstkrad(j) = stickrad
               gstkdefrgb(j) = usedefrgb
               gstkrgbf(1,j) = rgb(1)
               gstkrgbf(2,j) = rgb(2)
               gstkrgbf(3,j) = rgb(3)
               gstkrgbf(4,j) = rgb(4)
               gstktext(j) = itexture
            enddo
         endif
      enddo
c
c.......................................................................
 2    continue
c.......................................................................
c
c.....Transform the points to cartesian coordinates:
c
      do i = oldpt + 1, gnpt
         gxpt(i) = gxpt(i) + xct
         gypt(i) = gypt(i) + yct
         gzpt(i) = gzpt(i) + zct
         gxpt(i) = gxpt(i)*crys2car(1) + gypt(i)*crys2car(2) +
     &             gzpt(i)*crys2car(3)
         gypt(i) = gypt(i)*crys2car(4) + gzpt(i)*crys2car(5)
         gzpt(i) = gzpt(i)*crys2car(6)
      enddo
c
 800  format (/
     & 1x, 'MOLWS: Adding the Wigner-Seitz cell frame.')
 805  format (
     & 1x, 'Vertices of the WS polyhedron (crystal coordinates): ', i3)
 810  format (1x, i3, 3f10.4)
 815  format (
     & 1x, 'Faces of the WS polyhedron: ', i3/
     & 1x, 'number nsides isides ...')
 820  format (1x, 2i4, ' -> ', 20i3)
 825  format (/
     & 1x, 'Irreducible part of the WS polyhedron'/
     & 1x, 'Irreducible faces: ', 20i3)
 830  format (/
     & 1x, 'Number of irreducible tetrahedra: ', i4/
     & 1x, 'Crystal coordinates of the irreducible tetrahedra:')
 835  format (1x, i2, ' ->', 3(' (', 3f6.3, ')'))
c
      end
