c
c-----------------------------------------------------------------------
c
      subroutine read3dlines (lu3dl,         !Logical unit of 3DL file
     &                        con_mode,      !Connect mode
     &                        defrgb,        !Default rgbf
     &                        deflinerad,    !Default radius for lines
     &                        defpointrad,   !Default radius for points
     &                        uout)          !Printout unit
c
c.....read3dlines - read in 3DLINES data files and stores dots and lines
c     in the molecular graphics elements data base.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'molge.inc'
      include           'strtools.inc'
c
      integer           lu3dl, uout, con_mode
      real*8            defrgb(4), deflinerad, defpointrad
c
      integer           con_dots, con_cylinders, con_both
      parameter         (con_dots=10, con_cylinders=11, con_both=12)
c
      real*8            rgb(4), linerad, pointrad
      integer           MPT
      parameter         (MPT = 1000)
      logical           in_line
      integer           npt
      real*8            xpt(MPT), ypt(MPT), zpt(MPT)
      integer           num_point, num_line
c
      integer           i
      integer           lp, lw
      real*8            xx, yy, zz
      character*(600)   line
      character*(80)    word
      logical           usedefrgb, usedefrad
      logical           doagain, doread, ok, ok1, ok2
c
      integer           leng
      logical           setdble, setint
      character*(80)    uppcase, getword
      logical           finputline
c
      write (uout,800)
c
      in_line = .false.
      num_point = 0
      num_line = 0
      doagain = finputline (lu3dl, line)
c
c.......................................................................
      do while (doagain)
c.......................................................................
c
         lp = 1
         word = uppcase (getword (word, line, lp))
         lw = leng(word)
c
c.......................................................................
         if (word(1:1) .eq. '#') then
c.......................................................................
c
            continue
c
c.......................................................................
         else if (word(1:lw) .eq. 'POINT') then
c.......................................................................
c
            ok = setdble (xx, line, lp)
            ok = ok .and. setdble (yy, line, lp)
            ok = ok .and. setdble (zz, line, lp)
            usedefrgb = .true.
            usedefrad = .true.
            word = uppcase (getword (word, line, lp))
            lw = leng(word)
            doread = ok .and. lw.gt.0
            do while (doread)
               if (word(1:lw) .eq. 'RGB') then
                  ok1 = setdble (rgb(1), line, lp)
                  ok1 = ok1 .and. setdble (rgb(2), line, lp)
                  ok1 = ok1 .and. setdble (rgb(3), line, lp)
                  ok2 = setdble (rgb(4), line, lp)
                  if (ok1) usedefrgb = .false.
               else if (word(1:lw) .eq. 'RADIUS') then
                  ok1 = setdble (pointrad, line, lp)
                  if (ok1) usedefrad = .false.
               endif
               word = uppcase (getword (word, line, lp))
               lw = leng(word)
               doread = lw.gt.0
            enddo
            gnpt = gnpt + 1
            if (gnpt .gt. GMPT) call error ('read3dlines',
     &         'Too many points. Increase GMPT!', faterr)
            gxpt(gnpt) = xx
            gypt(gnpt) = yy
            gzpt(gnpt) = zz
            gnball = gnball + 1
            if (gnball .gt. GMBALL) call error ('read3dlines',
     &         'Too many balls. Increase GMBALL!', faterr)
            gballct(gnball) = gnpt
            if (usedefrad) then
               gballrad(gnball) = defpointrad
            else
               gballrad(gnball) = pointrad
            endif
            if (usedefrgb) then
               gballrgbf(1,gnball) = defrgb(1)
               gballrgbf(2,gnball) = defrgb(2)
               gballrgbf(3,gnball) = defrgb(3)
               gballrgbf(4,gnball) = defrgb(4)
            else
               gballrgbf(1,gnball) = rgb(1)
               gballrgbf(2,gnball) = rgb(2)
               gballrgbf(3,gnball) = rgb(3)
               gballrgbf(4,gnball) = rgb(4)
            endif
            num_point = num_point + 1
c
c.......................................................................
         else if (word(1:lw) .eq. 'LINE') then
c.......................................................................
c
            if (in_line) call error ('read3dlines',
     &         'file error: new LINE found before ending last one',
     &         faterr)
            in_line = .true.
            npt = 0
            usedefrgb = .true.
            usedefrad = .true.
            word = uppcase (getword (word, line, lp))
            lw = leng(word)
            doread = lw.gt.0
            do while (doread)
               if (word(1:lw) .eq. 'RGB') then
                  ok1 = setdble (rgb(1), line, lp)
                  ok1 = ok1 .and. setdble (rgb(2), line, lp)
                  ok1 = ok1 .and. setdble (rgb(3), line, lp)
                  ok2 = setdble (rgb(4), line, lp)
                  if (ok1) usedefrgb = .false.
               else if (word(1:lw) .eq. 'RADIUS') then
                  ok1 = setdble (linerad, line, lp)
                  if (ok1) usedefrad = .false.
               endif
               word = uppcase (getword (word, line, lp))
               lw = leng(word)
               doread = lw.gt.0
            enddo
            num_line = num_line + 1
c
c.......................................................................
         else if (word(1:lw) .eq. 'ENDLINE') then
c.......................................................................
c
            if (in_line .and. npt.gt.0) then
               if (con_mode.eq.con_cylinders .or.
     &             con_mode.eq.con_both           ) then
                  gnpt = gnpt + 1
                  if (gnpt .gt. GMPT) call error ('read3dlines',
     &               'Too many points. Increase GMPT!', faterr)
                  gxpt(gnpt) = xpt(1)
                  gypt(gnpt) = ypt(1)
                  gzpt(gnpt) = zpt(1)
                  do i = 2, npt
                     gnpt = gnpt + 1
                     if (gnpt .gt. GMPT) call error ('read3dlines',
     &                  'Too many points. Increase GMPT!', faterr)
                     gxpt(gnpt) = xpt(i)
                     gypt(gnpt) = ypt(i)
                     gzpt(gnpt) = zpt(i)
                     gnstk = gnstk + 1
                     if (gnstk .gt. GMSTK) call error ('read3dlines',
     &                  'Too many sticks. Increase GMSTK!', faterr)
                     gstkpt(1,gnstk) = gnpt - 1
                     gstkpt(2,gnstk) = gnpt
                     if (usedefrad) then
                        gstkrad(gnstk) = deflinerad
                     else
                        gstkrad(gnstk) = linerad
                     endif
                     if (usedefrgb) then
                        gstkdefrgb(gnstk) = .true.
                     else
                        gstkdefrgb(gnstk) = .false.
                        gstkrgbf(1,gnstk) = rgb(1)
                        gstkrgbf(2,gnstk) = rgb(2)
                        gstkrgbf(3,gnstk) = rgb(3)
                        gstkrgbf(4,gnstk) = rgb(4)
                     endif
                  enddo
               endif
               if (con_mode.eq.con_dots .or.
     &             con_mode.eq.con_both     ) then
                  do i = 1, npt
                     gnpt = gnpt + 1
                     if (gnpt .gt. GMPT) call error ('read3dlines',
     &                  'Too many points. Increase GMPT!', faterr)
                     gxpt(gnpt) = xpt(i)
                     gypt(gnpt) = ypt(i)
                     gzpt(gnpt) = zpt(i)
                     gnball = gnball + 1
                     if (gnball .gt. GMBALL) call error ('read3dlines',
     &                  'Too many balls. Increase GMBALL!', faterr)
                     gballct(gnball) = gnpt
                     if (usedefrad) then
                        gballrad(gnball) = deflinerad
                     else
                        gballrad(gnball) = linerad
                     endif
                     if (usedefrgb) then
                        gballrgbf(1,gnball) = defrgb(1)
                        gballrgbf(2,gnball) = defrgb(2)
                        gballrgbf(3,gnball) = defrgb(3)
                        gballrgbf(4,gnball) = defrgb(4)
                     else
                        gballrgbf(1,gnball) = rgb(1)
                        gballrgbf(2,gnball) = rgb(2)
                        gballrgbf(3,gnball) = rgb(3)
                        gballrgbf(4,gnball) = rgb(4)
                     endif
                  enddo
               endif
            else
               call error ('read3dlines',
     &            'ENDLINE error or line with no points', warning)
            endif
            in_line = .false.
c
c.......................................................................
         else if (word(1:lw) .eq. 'END') then
c.......................................................................
c
            doagain = .false.
c
c.......................................................................
         else
c.......................................................................
c
            if (in_line) then
               lp = 1
               ok = setdble (xx, line, lp)
               ok = ok .and. setdble (yy, line, lp)
               ok = ok .and. setdble (zz, line, lp)
               if (ok) then
                  npt = npt + 1
                  if (npt.gt.MPT) call error ('read3dlines',
     &               'Too many points in a LINE. Increase MPT', faterr)
                  xpt(npt) = xx
                  ypt(npt) = yy
                  zpt(npt) = zz
               else
                  call error ('read3dlines',
     &               'Error while reading a LINE', faterr)
               endif
            endif
c
c.......................................................................
         endif
c.......................................................................
c
c.......................................................................
         if (doagain) doagain = finputline (lu3dl, line)
      enddo
c.......................................................................
c
      write (uout,810) num_point, num_line
c
c.....FORMATS:
c
 800  format (/
     & 1x, 'READ3DLINES: Reading 3Dlines file.')
 810  format (
     & 1x, 'Points and lines read: ', 2i5)
c
      end
