c
c-----------------------------------------------------------------------
c
      subroutine readdbasin (isodensity, datafile, uout)
c
c.....readdbasin - read the polyhedron description as an DBASIN file.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'polyhedron.inc'
      include           'strtools.inc'
c
      real*8            isodensity
      character*(*)     datafile
      integer           uout
c
      integer           luo, i, j, k, ndel
c
      integer           mpoint
      parameter         (mpoint = 500)
      real*8            lrho(0:500), lisoden
      real*8            x, y, z, x1, y1, z1
      real*8            rdist, rdelta, rr, theta, phi, st
*
      real*8            riso(mvert), rbasin(mvert), ract(mvert)
      real*8            xyz1(3), xyz2(3), xyz3(3), xyz4(3)
      real*8            xyz5(3), xyz6(3), xyz7(3), xyz8(3)
      integer           i1, i2, i3
*
      integer           nedge, npoint, nerr
      logical           done, dolog, del
      character*(80)    line
c
      real*8            xnuc, ynuc, znuc
      common /rdb1/     xnuc, ynuc, znuc
c
      logical           commentline
      integer           leng, fopen
c
c.....Open the input file:
c
      if (fopen (luo, datafile, ioread) .eq. ioerror) then
         write (uout,*) 'readdbasin: Data file is ',
     &               datafile(1:leng(datafile))
         call error ('readdbasin', 'I/O error when opening points file',
     &               faterr)
         return
      endif
c
      dolog = loginterpol
      nvf = 0
      isw1 = 0
c
c.....Skip initial comment lines, if any:
c
      do while (commentline(luo, line))
         continue
      enddo
c
c.....Start reading:
c
      read (luo,*) nvert, nface(isw1), nedge
      read (luo,*) npoint, xnuc, ynuc, znuc, lrho(0)
      if (npoint .gt. mpoint) call error ('readdbasin',
     &   'Too many density values per point. Increase MPOINT!', faterr)
      if (dolog) then
         lrho(0) = log(lrho(0))
         lisoden = log(isodensity)
      else
         lisoden = isodensity
      endif
c
      do j = 1, nvert
c
c........Get the vertex coordinates and the density values:
c
         read (luo,*) x, y, z, (lrho(k), k = 1, npoint)
         if (dolog) then
            do k = 1, npoint
               lrho(k) = log(lrho(k))
            enddo
         endif
c
c........Pass the coordinates to spherical:
c
         x1 = x - xnuc
         y1 = y - ynuc
         z1 = z - znuc
         rdist = sqrt(x1*x1 + y1*y1 + z1*z1)
         rdelta = rdist / npoint
         theta = acos(z1 / rdist)
         phi = atan2(y1, x1)
         st = sin(theta)
         sthcph(j) = st * cos(phi)
         sthsph(j) = st * sin(phi)
         cth(j) = cos(theta)
c
c........Get the r coordinate for the input density value. 
c        Interpolate the density values for this ray:
c
         k = 1
         done = .false.
         do while (k.le.npoint .and. .not.done)
            if ((lrho(k-1)-lisoden)*(lrho(k)-lisoden) .le. 0d0) then
               rr = rdelta * ( (k-1) +
     &                     (lisoden-lrho(k-1)) / (lrho(k)-lrho(k-1)) )
               done = .true.
            else
               k = k + 1
            endif
         enddo
         if (.not. done) then
            nerr = nerr + 1
            rr = rdist
         endif
         xvert(j) = xnuc + rr * sthcph(j)
         yvert(j) = ynuc + rr * sthsph(j)
         zvert(j) = znuc + rr * cth(j)
c
c........Prepare the data to interpolate the vertices if clipping is
c        activated
c
         if (basinclip) then
            rbasin(j) = rdist
            ract(j) = rr
            if (done) then
               deletevert(j) = .false.
               riso(j) = rr
            else
               riso(j) = rdelta * ( (npoint-1) +
     &                   (lisoden-lrho(npoint-1)) /
     &                   (lrho(npoint)-lrho(npoint-1)) )
               if (riso(j) .lt. rdist) then
                  riso(j) = rdist * (lisoden-lrho(0)) /
     &                      (lrho(npoint)-lrho(0))
                  if (riso(j) .lt. rdist) call error ('readdbasin',
     &               'trouble extrapolating isosurface', warning)
               endif
               deletevert(j) = .true.
            endif
         endif
      enddo
c
c.....Read in the faces:
c
      do j = 1, nface(isw1)
         read (luo,*) nvf, (iface(k,j,isw1), k = 1, nvf)
      enddo
      do k = 1, nvf
         do j = 1, nface(isw1)
            iface(k,j,isw1) = iface(k,j,isw1) + 1
         enddo
      enddo
c
      call fclose (luo)
c
      write (uout,100) datafile(1:leng(datafile)), nvert, nface(isw1),
     &                 nvf, isodensity, nerr, basinclip
c
      if (.not.basinclip .or. nerr.le.0) return
c
c-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
c
c.....CLIPPING of the iso surface to the basin surface.
c     New vertices will be interpolated.
c
c.....To simplify the analysis, the faces will be reduced to triangles:
c
      isw0 = mod(isw1+1,2)
      nface(isw0) = 0
      do i = 1, nface(isw1)
         del = .true.
         do k = 1, nvf
            del = del .and. deletevert(iface(k,i,isw1))
         enddo
         if (.not. del) then
            do k = 2, nvf-1
               nface(isw0) = nface(isw0) + 1
               iface(1,nface(isw0),isw0) = iface(1,i,isw1)
               iface(2,nface(isw0),isw0) = iface(k,i,isw1)
               iface(3,nface(isw0),isw0) = iface(k+1,i,isw1)
            enddo
         endif
      enddo
      nvf = 3
*mdc*if debug
      write (uout,105) nvert, nface(isw0), nvf
*mdc*endif
c
c.....Run now over the triangular faces. For each face determine how
c     many vertices have been clipped to the basin surface:
c
      nface(isw1) = 0
      do i = 1, nface(isw0)
         ndel = 0
         if (deletevert(iface(1,i,isw0))) ndel = ndel + 1
         if (deletevert(iface(2,i,isw0))) ndel = ndel + 1
         if (deletevert(iface(3,i,isw0))) ndel = ndel + 1
*mdc*if debug
         write (uout,*) 'face,ndel ', i, ndel
*mdc*endif
         if (ndel .eq. 0) then
c
c...........Save this face if none of its vertices have been clipped:
c
            nface(isw1) = nface(isw1) + 1
            iface(1,nface(isw1),isw1) = iface(1,i,isw0)
            iface(2,nface(isw1),isw1) = iface(2,i,isw0)
            iface(3,nface(isw1),isw1) = iface(3,i,isw0)
         else if (ndel .eq. 1) then
c
c...........If one vertex has been clipped, two new points appear and
c           the triangular face produces two triangles:
c
            if (deletevert(iface(1,i,isw0))) i1 = 1
            if (deletevert(iface(2,i,isw0))) i1 = 2
            if (deletevert(iface(3,i,isw0))) i1 = 3
            i2 = mod(i1,3) + 1
            i3 = mod(i2,3) + 1
            i1 = iface(i1,i,isw0)
            i2 = iface(i2,i,isw0)
            i3 = iface(i3,i,isw0)
            xyz1(1) = xvert(i1)
            xyz1(2) = yvert(i1)
            xyz1(3) = zvert(i1)
            xyz2(1) = xvert(i2)
            xyz2(2) = yvert(i2)
            xyz2(3) = zvert(i2)
            xyz3(1) = xvert(i3)
            xyz3(2) = yvert(i3)
            xyz3(3) = zvert(i3)
            xyz8(1) = xnuc + riso(i1) * sthcph(i1)
            xyz8(2) = ynuc + riso(i1) * sthsph(i1)
            xyz8(3) = znuc + riso(i1) * cth(i1)
            xyz4(1) = xnuc + rbasin(i2) * sthcph(i2)
            xyz4(2) = ynuc + rbasin(i2) * sthsph(i2)
            xyz4(3) = znuc + rbasin(i2) * cth(i2)
            xyz5(1) = xnuc + rbasin(i3) * sthcph(i3)
            xyz5(2) = ynuc + rbasin(i3) * sthsph(i3)
            xyz5(3) = znuc + rbasin(i3) * cth(i3)
*            call gxyz (xyz8, xvert(i1),yvert(i1),zvert(i1), riso(i1))
*            call gxyz (xyz4, xvert(i2),yvert(i2),zvert(i2), rbasin(i2))
*            call gxyz (xyz5, xvert(i3),yvert(i3),zvert(i3), rbasin(i3))
*            call gcut (xyz6, xyz2, xyz8, xyz1, xyz4)
*            call gcut (xyz7, xyz3, xyz8, xyz1, xyz5)
            call gcut2 (xyz6, xyz1, xyz4, xyz5, xyz2, xyz8)
            call gcut2 (xyz7, xyz1, xyz4, xyz5, xyz3, xyz8)
            nface(isw1) = nface(isw1) + 1
            iface(1,nface(isw1),isw1) = i2
            iface(2,nface(isw1),isw1) = nvert + 2
            iface(3,nface(isw1),isw1) = nvert + 1
            nface(isw1) = nface(isw1) + 1
            iface(1,nface(isw1),isw1) = i2
            iface(2,nface(isw1),isw1) = i3
            iface(3,nface(isw1),isw1) = nvert + 2
            nvert = nvert + 1
            xvert(nvert) = xyz6(1)
            yvert(nvert) = xyz6(2)
            zvert(nvert) = xyz6(3)
            nvert = nvert + 1
            xvert(nvert) = xyz7(1)
            yvert(nvert) = xyz7(2)
            zvert(nvert) = xyz7(3)
         else if (ndel .eq. 2) then
c
c...........If two vertices have been clipped, two new points appear and
c           the triangular face produces one triangle:
c
            if (deletevert(iface(1,i,isw0)) .and.
     &          deletevert(iface(2,i,isw0))) then
               i1 = iface(1,i,isw0)
               i2 = iface(2,i,isw0)
               i3 = iface(3,i,isw0)
            else if (deletevert(iface(2,i,isw0)) .and.
     &               deletevert(iface(3,i,isw0))) then
               i1 = iface(2,i,isw0)
               i2 = iface(3,i,isw0)
               i3 = iface(1,i,isw0)
            else if (deletevert(iface(3,i,isw0)) .and.
     &               deletevert(iface(1,i,isw0))) then
               i1 = iface(3,i,isw0)
               i2 = iface(1,i,isw0)
               i3 = iface(2,i,isw0)
            else
               call error ('readdbasin', 'Check clipping algorithm!',
     &                     faterr)
            endif
            xyz1(1) = xvert(i1)
            xyz1(2) = yvert(i1)
            xyz1(3) = zvert(i1)
            xyz2(1) = xvert(i2)
            xyz2(2) = yvert(i2)
            xyz2(3) = zvert(i2)
            xyz3(1) = xvert(i3)
            xyz3(2) = yvert(i3)
            xyz3(3) = zvert(i3)
            xyz4(1) = xnuc + riso(i1) * sthcph(i1)
            xyz4(2) = ynuc + riso(i1) * sthsph(i1)
            xyz4(3) = znuc + riso(i1) * cth(i1)
            xyz5(1) = xnuc + riso(i2) * sthcph(i2)
            xyz5(2) = ynuc + riso(i2) * sthsph(i2)
            xyz5(3) = znuc + riso(i2) * cth(i2)
            xyz6(1) = xnuc + rbasin(i3) * sthcph(i3)
            xyz6(2) = ynuc + rbasin(i3) * sthsph(i3)
            xyz6(3) = znuc + rbasin(i3) * cth(i3)
*            call gxyz (xyz4, xvert(i1),yvert(i1),zvert(i1), riso(i1))
*            call gxyz (xyz5, xvert(i2),yvert(i2),zvert(i2), riso(i2))
*            call gxyz (xyz6, xvert(i3),yvert(i3),zvert(i3), rbasin(i3))
*mdc*if debug
*            write (uout,*) '1: ', xyz1(1), xyz1(2), xyz1(3)
*            write (uout,*) '2: ', xyz2(1), xyz2(2), xyz2(3)
*            write (uout,*) '3: ', xyz3(1), xyz3(2), xyz3(3)
*            write (uout,*) '4: ', xyz4(1), xyz4(2), xyz4(3)
*            write (uout,*) '5: ', xyz5(1), xyz5(2), xyz5(3)
*            write (uout,*) '6: ', xyz6(1), xyz6(2), xyz6(3)
*mdc*endif
*            call gcut (xyz7, xyz3, xyz4, xyz1, xyz6)
            call gcut2 (xyz7, xyz1, xyz2, xyz6, xyz3, xyz4)
*mdc*if debug
*            write (uout,*) '7: ', xyz7(1), xyz7(2), xyz7(3)
*mdc*endif
*            call gcut (xyz8, xyz3, xyz5, xyz2, xyz6)
            call gcut2 (xyz8, xyz1, xyz2, xyz6, xyz3, xyz5)
*mdc*if debug
*            write (uout,*) '8: ', xyz8(1), xyz8(2), xyz8(3)
*mdc*endif
            nface(isw1) = nface(isw1) + 1
            iface(1,nface(isw1),isw1) = i3
            iface(2,nface(isw1),isw1) = nvert + 1
            iface(3,nface(isw1),isw1) = nvert + 2
            nvert = nvert + 1
            xvert(nvert) = xyz7(1)
            yvert(nvert) = xyz7(2)
            zvert(nvert) = xyz7(3)
            nvert = nvert + 1
            xvert(nvert) = xyz8(1)
            yvert(nvert) = xyz8(2)
            zvert(nvert) = xyz8(3)
         else
c
c...........Remove the face if the three vertices have been clipped:
c
            continue
         endif
      enddo
      write (uout,105) nvert, nface(isw1), nvf
c
 100  format (/
     & 1x, 'READDBASIN: Reading start points from a file'/
     & 1x, 'Data file name:    ', a/
     & 1x, 'Vertices read in:  ', i6/
     & 1x, 'Number of faces:   ', i6/
     & 1x, 'Vertices per face: ', i6/
     & 1x, 'Isodensity surface: ', e14.6/
     & 1x, 'Fails to determine isodensity point: ', i4/
     & 1x, 'Clip the points with the basin surface? ', l4)
 105  format (1x, 'CLIPPING -----'/
     & 1x, 'Vertices after cliping: ', i6/
     & 1x, 'Faces after cliping:    ', i6/
     & 1x, 'Vertices per face:      ', i6)
c
      end
c
c-----------------------------------------------------------------------
c
      subroutine gxyz (xyz, x,y,z, r)
c
c.....gxyz - returns the cartesian coordinates of a point in the 
c     direction from (xnuc,ynuc,znuc) to (x,y,z) such that the distance
c     to (xnuc,ynuc,znuc) is r.
c
      include           'implicit.inc'
      real*8            xyz(3), x, y, z, r
c
      real*8            xnuc, ynuc, znuc
      common /rdb1/     xnuc, ynuc, znuc
c
      real*8            x1, y1, z1, rdist, theta, phi, st
c
      x1 = x - xnuc
      y1 = y - ynuc
      z1 = z - znuc
      rdist = sqrt(x1*x1 + y1*y1 + z1*z1)
      theta = acos(z1 / rdist)
      phi = atan2(y1, x1)
      st = sin(theta)
      xyz(1) = xnuc + r * st * cos(phi)
      xyz(2) = ynuc + r * st * sin(phi)
      xyz(3) = znuc + r * cos(theta)
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine gcut (xyz, xyz1, xyz2, xyz3, xyz4)
c
c.....gcut - gets the cross point between the two straight segments
c     (xyz1 --> xyz2) and (xyz3 --> xyz4).
c
      include           'implicit.inc'
      include           'error.inc'
      real*8            xyz(3), xyz1(3), xyz2(3), xyz3(3), xyz4(3)
c
      real*8            t, t1, t2, t3, den
c
      den = xyz2(1) - xyz1(1) - xyz4(1) + xyz3(1)
      if (abs(den) .gt. 1d-5) then
         t1 = (xyz3(1) - xyz1(1)) / den
      else
         call error ('gcut', 'Undefined x cross', warning)
         t1 = 0.5d0
      endif
c
      den = xyz2(2) - xyz1(2) - xyz4(2) + xyz3(2)
      if (abs(den) .gt. 1d-5) then
         t2 = (xyz3(2) - xyz1(2)) / den
      else
         call error ('gcut', 'Undefined y cross', warning)
         t2 = 0.5d0
      endif
c
      den = xyz2(3) - xyz1(3) - xyz4(3) + xyz3(3)
      if (abs(den) .gt. 1d-5) then
         t3 = (xyz3(3) - xyz1(3)) / den
      else
         call error ('gcut', 'Undefined z cross', warning)
         t3 = 0.5d0
      endif
c
      t = (t1+t2+t3) / 3d0
      if (abs(t-t1).gt.1d-5 .or. abs(t-t2).gt.1d-5 .or.
     &    abs(t-t3).gt.1d-5) then
         call error ('gcut', 'Irregular cross', warning)
*mdc*if debug
         write (0,*) 't: ', t1, t2, t3, t
*mdc*endif
         xyz(1) = xyz1(1) + t1 * (xyz2(1) - xyz1(1))
         xyz(2) = xyz1(2) + t2 * (xyz2(2) - xyz1(2))
         xyz(3) = xyz1(3) + t3 * (xyz2(3) - xyz1(3))
      else
         xyz(1) = xyz1(1) + t * (xyz2(1) - xyz1(1))
         xyz(2) = xyz1(2) + t * (xyz2(2) - xyz1(2))
         xyz(3) = xyz1(3) + t * (xyz2(3) - xyz1(3))
      endif
c
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine gcut2 (xyz, xyz1, xyz2, xyz3, xyz4, xyz5)
c
c.....gcut - gets the cross point between the plane {xyz1,xyz2,xyz3}
c     and the segment {xyz4,xyz5}.
c
      include           'implicit.inc'
      include           'error.inc'
      real*8            xyz(3), xyz1(3), xyz2(3), xyz3(3)
      real*8            xyz4(3), xyz5(3)
c
      real*8            x12, x13, x54, y12, y13, y54, z12, z13, z54
      real*8            a, b, c, d, t, den
c
      x12 = xyz2(1) - xyz1(1)
      y12 = xyz2(2) - xyz1(2)
      z12 = xyz2(3) - xyz1(3)
      x13 = xyz3(1) - xyz1(1)
      y13 = xyz3(2) - xyz1(2)
      z13 = xyz3(3) - xyz1(3)
      x54 = xyz4(1) - xyz5(1)
      y54 = xyz4(2) - xyz5(2)
      z54 = xyz4(3) - xyz5(3)
c
      a = y12*z13 - y13*z12
      b = z12*x13 - z13*x12
      c = x12*y13 - x13*y12
      d = - a*xyz1(1) - b*xyz1(2) - c*xyz1(3)
c
      den = a*x54 + b*y54 + c*z54
      if (abs(den) .lt. 1d-6) then
         call error ('gcut2', 'Error1', warning)
         t = 0.5d0
      else
         t = (a*xyz4(1)+b*xyz4(2)+c*xyz4(3)+d) /
     &       (a*x54 + b*y54 + c*z54)
         if (t.lt.0d0 .or. t.gt.1d0) then
            call error ('gcut2', 'Error1', warning)
            t = 0.5d0
         endif
      endif
      xyz(1) = xyz4(1) + t * (xyz5(1) - xyz4(1))
      xyz(2) = xyz4(2) + t * (xyz5(2) - xyz4(2))
      xyz(3) = xyz4(3) + t * (xyz5(3) - xyz4(3))
c
      return
      end
