c-----------------------------------------------------------------------
c
c     SSSSSSSS      PPPPPPPPPP        GGGGGGGG      RRRRRRRRRR
c    SSSSSSSSSS     PPPPPPPPPPP      GGGGGGGGGG     RRRRRRRRRRR
c   SSS             PPP      PPP    GGG      GGG    RRR      RRR
c   SSS             PPP      PPP    GGG      GGG    RRR      RRR
c    SSSSSSSSS      PPP      PPP    GGG             RRR      RRR
c     SSSSSSSSS     PPP      PPP    GGG             RRR      RRR
c            SSS    PPPPPPPPPPP     GGG   GGGGGG    RRRRRRRRRRR
c            SSS    PPPPPPPPPP      GGG   GGGGGG    RRRRRRRRRR
c   SSS      SSS    PPP             GGG      GGG    RRR    RRR
c   SSS      SSS    PPP             GGG      GGG    RRR     RRR
c    SSSSSSSSSS     PPP              GGGGGGGGGG     RRR      RRR
c     SSSSSSSS      PPP               GGGGGGGG      RRR       RRR
c
c.....Space group symbol recognition library.
c
c.....Based upon work by Allen C. Larson (the National Research
c     Council of Canada) in the sixties.
c
c.....Rewritten in Fortran 77, cleaned up and corrected by Angel Martin
c     Pendas (Universidad de Oviedo).
c
c----------------------------------------------------------------------
c
      subroutine spgr (spg, lpt, lptx, ier)
c
c.....interprets the space group symbol
c
c     data in the calling sequence are
c
c     spg ...... space group symbol
c     laue ..... output the laue group number.
c                1=1bar, 2=2/m, 3=mmm, 4=4/m, 5=4/mm, 6=r3r, 7=r3mr,
c                8=3, 9=3m1, 10=31m, 11=6/m, 12=6/mmm, 13=m3 AND 14=m3m
c     naxis .... unique axis in monoclinic space groups
c     ncent .... 1bar flag  (0/1) for (acentric/centric)
c     lcent .... lattice centering number.
c                1=P, 2=A, 3=B, 4=C, 5=I, 6=F and 7=R
c     neqv ..... Number of symmetry matrices generated.
c     jrt ...... neqv (3,4,neqv) matrices
c     cen ...... lattice centering vectors
c     ncv ...... Number of lattice centering vectors.
c     lpt ...... stdout
c                if (lpt.lt.0) no listing will be produced
c     lptx ..... stderr
c     ier ...... if (ier.ne.0). An error has been found.
c
c
      include           'spgcom.inc'
c
      character*(30)    spg
      character*(1)     chr(25)
c
      dimension         rt(5,4,25),d(3,3)
      dimension         l(4,4)
      dimension         lcen(7)
      character*(30)    uppcase
c
c.....data in lcen for  C B A P F I R  respectively
c
      data
     & lcen(1),lcen(2),lcen(3),lcen(4),lcen(5) /4,3,2,1,6/,
     & lcen(6),lcen(7)                         /5,7/
c
c                  1    2    3    4    5    6    7    8    9    10
      data chr  / ' ', 'C', 'B', 'A', 'P', 'F', 'I', 'R', 'M', 'N',
     &            'D', '1', '2', '3', '4', '5', '6', '-', '/', 'H',
     &            '.', ' ', ' ', ' ', ' ' /
c                  11   12   13   14   15   16   17   18   19   20
c
      do i = 1, 4
         do j = 1, 4
            l(i,j) = 0
         enddo
      enddo
      ier    = 0
      ncent  = 0
      laue   = 0
      naxis  = 0
      ierx   = 0
      n      = 0
      nromb  = 0
      ncubic = 0
c
c.....transform symbol to uppercase
c
      length = leng(spg)
      spg = uppcase(spg)
c
C.....break the space group symbol into the 4 fields and code
C     as numerical values for manipulation
      k = 0
      m = 0
      do 110 j=1,length
         ifound=0
         do 100 i=1,21
c...........Exit if found
            if ( spg(j:j).eq.chr(i) ) then 
                ifound=i
                go to 101
            endif
 100     continue
 101     if (ifound.ne.0) then 
            if (k+m+ifound.ne.1) then
                if ( ifound.ne.1) then 
                   if ( m.eq.0 ) k=k+1
                   m = m+1
                   l(m,k) = ifound
                else
                   m=0
c..................exit loop if more than 3 fields read
                   if (k.gt.3) goto 200
                endif
                if ( ifound.lt.12 .or. m.ge.4) then
                   m=0
c..................exit loop if more than 3 fields read
                   if (k.gt.3) goto 200
                endif
            endif
         endif      
 110  continue
c
c
  200 if ( k.le.1 .or. l(1,1).gt.8 ) then
         ier=1
         if (l(1,1).gt.8) ier=2
         write (6,*) ier
         if (lptx.ge.0) call sgerrs (spg(1:30), ier, lptx)
         return
      endif
c
c.....convert the -N notation to the nb(ar) notation
c
      nmenos=0
      if ( l(1,2).eq.18 ) then
         nmenos=nmenos+1
         call sglpak(l(1,2),ier)
         if (ier .gt. 0 ) then
            if (lptx.ge.0) call sgerrs (spg(1:30), ier, lptx)
            return
         endif
      endif
      if ( l(1,3).eq.18 ) then
         nmenos=nmenos+1
         call sglpak(l(1,3),ier)
         if (ier .gt. 0 ) then
            if (lptx.ge.0) call sgerrs (spg(1:30), ier, lptx)
            return
         endif
      endif
      if ( l(1,4).eq.18 ) then
         nmenos=nmenos+1
         call sglpak(l(1,4),ier)
         if (ier .gt. 0 ) then
            if (lptx.ge.0) call sgerrs (spg(1:30), ier, lptx)
            return
         endif
      endif
c
c.....only one bar allowed
c
      if ( nmenos.gt.1 ) then
         ier=24
         if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
         return
      endif
c
c.....set the matrix count n to 2
c
      n = 2
c
c.....set the translation flags
c
      d(1,1) = 0.0
      d(1,2) = 0.0
      d(1,3) = 0.0
      d(2,1) = 0.0
      d(2,2) = 0.0
      d(2,3) = 0.0
      d(3,1) = 0.0
      d(3,2) = 0.0
      d(3,3) = 0.0
c
c.....set the lattice centering flag. 1=P, 2=A, 3=B, 4=C, 5=I, 6=F, 7=R
c
      lcent = l(1,1)-1
      lcent = lcen(lcent)
      if ( lcent.eq.7 ) then
c
c........rhombohedral lattice.
c        make sure that there is a 3-axis.
c
         if ( l(1,2).ne.14 ) then
            ier=3
            if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
            return
         endif
         if ( l(1,k).ne.8 ) then
c
c...........hexagonal axes. retain r centering and set laue to 8 or 9
c
            if ( l(1,k).eq.20 ) k=k-1
            laue = k+6
         else
c
c...........rhombohedral axes. delete R centering.set laue to 6 or 7
c
            lcent = 1
            k = k-1
            laue = k+4
            nromb=1
         endif
      else
c
c........determine laue and some preliminary data
c
         ier = 0
         ncubic = 0
         call sglatc(k, l, d, lcent, laue, naxis, ier, ncubic, id)
         if ( ier.gt.0 ) then
            if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
            return
         endif
      endif
c
      if ( ncubic.eq.1 .or. nromb.eq.1 ) then
c
c.........cubic or rhombohedral cell. insert the 3-fold axis
c
          call sgrmat(rt(1,1,2),0,1,0,0,0,1,1,0,0)
          call sgrmat(rt(1,1,3),0,0,1,1,0,0,0,1,0)
          n = 4
      endif
c
      call sgrmat(rt,1,0,0,0,1,0,0,0,1)
c
c.....decode the last 3 fields of the symbol
c
      do 20 m=2,k
         if ( l(1,m).eq.0 ) then
            ier=6
            if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
            return
         endif
         i = iabs(l(1,m)-5)
  10     if ( i.le.0.or.i.gt.15 ) then
            ier=7
            if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
            return
         endif
         nxi = n
c
c       A   B   C   M   N   D   1   2   3   4   5   6   -   /   H
c  i=   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15
c
         if (i.ge.1 .and. i.le.5) then
c
c...........a mirror is needed
c
            if (m .eq. 2) then
c
C         A   B   C  AXIS
c  m=     1   2   3
c
               if ( laue.gt.3 )  then
                  if (i.eq.3 ) then
                      ier=10
                      if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
                      return
                  endif
c
c.................a C-axis mirror
c
                  call sgrmat(rt(1,1,n),1,0,0,0,1,0,0,0,-1)
                  rt(3,4,n) = d(3,3)
                  if ( i.eq.1.or.i.eq.5 ) rt(1,4,n) = 0.5
                  if ( i.eq.2.or.i.eq.5 ) rt(2,4,n) = 0.5
                  if ( m.eq. 2.and.l(1,2).eq.17) then
c
c....................if this is a 63-axis the mirror is at 1/4
c
                     if ( l(2,2).eq.14 ) rt(3,4,n)=0.5
                  endif
               else if (k.eq.2) then
                  if (i.eq.2 ) then
                     ier=9
                     if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
                     return
                  endif
c
c.................a B-axis mirror
c
                  call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,1)
                  rt(2,4,n) = d(2,2)
                  if ( i.eq.1.or.i.eq.5 ) rt(1,4,n) = 0.5
                  if ( i.eq.3.or.i.eq.5 ) rt(3,4,n) = 0.5
               else if (i.eq.1) then
                  ier=8
                  if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
                  return
               else
c
c.................a A-axis mirror
c
                  call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,1)
                  rt(1,4,n) = d(1,1)
                  if ( i.eq.2.or.i.eq.5 ) rt(2,4,n)=0.5
                  if ( i.eq.3.or.i.eq.5 ) rt(3,4,n)=0.5
               endif
            else if (m.eq.3) then
               if ( l(1,2).eq.14.or.l(1,2).eq.17 ) then
c
                  if ( laue.eq.7 ) then
c
c....................a diagonal mirrror perpendicular to -110
c
                     call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,1)
                     rt(1,4,n) = d(2,2)
                     rt(2,4,n) = -d(2,2)
                     if ( i.eq.3.or.i.eq.5 ) rt(3,4,n) = 0.5
                     if (( laue.ne.7 .or. i.ne.3) .and.
     &                   (  i.ne.3 .and. i.ne.4  )) then
                        if ( lcent.eq.6.or.lcent.eq.4 ) then
c
c.......................F or C-centered tetragonal. glides are 1/4,1/4
c
                           rt(1,4,n) = 0.25+rt(1,4,n)
                           rt(2,4,n) = 0.25+rt(2,4,n)
                        else
                           rt(1,4,n) = 0.5+rt(1,4,n)
                           rt(2,4,n) = 0.5+rt(2,4,n)
                        endif
                     endif
                  else
c
c....................mirror normal to (1000) in hex cell
c
                     call sgrmat(rt(1,1,n),-1,1,0,0,1,0,0,0,1)
                     if ( i.eq.3 ) rt(3,4,n)=0.5
                  endif
               else if (l(1,2).eq.15) then
                  if (i.eq.1) then
                     ier=8
                     if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
                     return
                  else
c
c...................a A-axis mirror
c
                    call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,1)
                    rt(1,4,n) = d(1,1)
                    if ( i.eq.2.or.i.eq.5 ) rt(2,4,n)=0.5
                    if ( i.eq.3.or.i.eq.5 ) rt(3,4,n)=0.5
                  endif
               else
                  if (i.eq.2 ) then
                     ier=9
                     if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
                     return
                  endif
c
c.................a B-axis mirror
c
                  call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,1)
                  rt(2,4,n) = d(2,2)
                  if ( i.eq.1.or.i.eq.5 ) rt(1,4,n) = 0.5
                  if ( i.eq.3.or.i.eq.5 ) rt(3,4,n) = 0.5
               endif
            else if (m.eq.4) then
c
c..............it is not cubic or tetragonal nor trigonal or hexagonal
c
               if (( l(1,3).eq.14.or.l(1,2).eq.15 ) .or.
     &             ( l(1,2).eq.14.or.l(1,2).eq.17 )) then
c
c.................a diagonal mirrror perpendicular to -110
c
                  call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,1)
                  rt(1,4,n) = d(2,2)
                  rt(2,4,n) = -d(2,2)
                  if ( i.eq.3.or.i.eq.5 ) rt(3,4,n) = 0.5
                  if (( laue.ne.7 .or. i.ne.3) .and.
     &                (  i.ne.3 .and. i.ne.4  )) then
                     if ( lcent.eq.6.or.lcent.eq.4 ) then
c
c....................F or C-centered tetragonal. glides are 1/4,1/4
c
                        rt(1,4,n) = 0.25+rt(1,4,n)
                        rt(2,4,n) = 0.25+rt(2,4,n)
                     else
                        rt(1,4,n) = 0.5+rt(1,4,n)
                        rt(2,4,n) = 0.5+rt(2,4,n)
                     endif
                  endif
               else
                  if ( i.eq.3 ) then
                     ier=10
                     if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
                     return
                  endif
c
c.................a c-axis mirror
c
                  call sgrmat(rt(1,1,n),1,0,0,0,1,0,0,0,-1)
                  rt(3,4,n) = d(3,3)
                  if ( i.eq.1.or.i.eq.5 ) rt(1,4,n) = 0.5
                  if ( i.eq.2.or.i.eq.5 ) rt(2,4,n) = 0.5
                  if ( m.eq. 2.and.l(1,2).eq.17) then
c
c....................if this is a 63-axis the mirror is at 1/4
c
                     if ( l(2,2).eq.14 ) rt(3,4,n)=0.5
                  endif
               endif
            endif
         else if (i.eq.6 ) then
c
c...........d type mirror
c
            if ( lcent.le.1 ) then
               ier=11
               if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
               return
            endif
c
c           go to (500,246,247,248),m
c
            if (m.eq.2) then
               if ( laue.gt.3 ) then
                  call sgrmat(rt(1,1,n),1,0,0,0,1,0,0,0,-1)
                  rt(1,4,n) = 0.25
                  rt(2,4,n) = 0.25
                  if ( id.eq.2 ) rt(3,4,n)=0.25
               else if (k.eq.2) then
                  call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,1)
                  rt(1,4,n) = 0.25
                  if ( id.eq.2 ) rt(2,4,n)=0.25
                  if ( laue.eq.5 ) rt(2,4,n) = d(2,1)
                  rt(3,4,n) = 0.25
               else
                  call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,1)
                  if ( id.eq.2 ) rt(1,4,n)=0.25
                  rt(2,4,n) = 0.25
                  rt(3,4,n) = 0.25
               endif
            else if (m.eq.3) then
               call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,1)
               rt(1,4,n) = 0.25
               if ( id.eq.2 ) rt(2,4,n)=0.25
               if ( laue.eq.5 ) rt(2,4,n) = d(2,1)
               rt(3,4,n) = 0.25
            else if (m.eq.4) then
               if ( l(1,2).eq.15.or.l(1,3).eq.14 ) then
c
c.................cubic or tetragonal. d-glide along diagonal
c
                  call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,1)
                  rt(1,4,n) = 0.25
                  rt(2,4,n) = 0.25
                  rt(3,4,n) = 0.25
                  if (l(1,3).eq.13) then
                     rt(1,4,n) = 0.0
                     rt(2,4,n) = 0.5
                  endif
               else
c
c.................it is not tetragonal or cubic
c
                  call sgrmat(rt(1,1,n),1,0,0,0,1,0,0,0,-1)
                  rt(1,4,n) = 0.25
                  rt(2,4,n) = 0.25
                  if ( id.eq.2 ) rt(3,4,n)=0.25
               endif
            endif
         else if (i.eq.7) then
c
c...........1 fold rotation
c
            if ( l(2,m).eq.3 ) then
c
c..............we have a center of symmetry
c
               ncent = 1
            endif
c
c...........Loop exit
c
            goto 20
c
         else if (i.eq.8) then
c
c...........2 fold rotation axis
c...........we will not allow a -2 axis.
c
            if ( l(2,m).eq.3 )then
               ier=19
               if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
               return
            endif
c
c           go to (500,256,257,258),m
c
            if (m.eq.2) then
               if ( k.eq.2 )then
c
c.................rotation about the b-axis
c
                  call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,-1)
                  rt(1,4,n) = d(1,2)
                  rt(3,4,n) = d(3,2)
                  if ( l(2,m).eq.12 ) rt(2,4,n)=0.5
               else
c
c..................rotation about the a-axis. (orthogonal cell)
c
                   call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,-1)
                   rt(2,4,n) = d(2,1)
                   rt(3,4,n) = d(3,1)
                   if ( iabs(l(2,m)-13).eq.1 ) rt(1,4,n) = 0.5
               endif
            else if (m.eq.3) then
               if ( l(1,2).eq.14 .or. l(1,2).eq.17 ) then
                  if (l(1,2).eq.17 .and. l(1,4).ne.12) then
c
c.....................2-axis normal to (-2110)
c.....................used for the p 6n22 groups
c
                      call sgrmat(rt(1,1,n),1,-1,0,0,-1,0,0,0,-1)
                  else
                     if ( laue.eq.7 ) then
c
c........................2-axis normal to (110)
c
                         call sgrmat(rt(1,1,n),0,-1,0,-1,0,0,0,0,-1)
                     else
c
c........................2-axis along to (11-20) trig  and (110) tetrag
c........................used for the p 3n21 groups
c
                         call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,-1)
                         rt(1,4,n) = d(2,1)
                         if ( l(2,m).eq.12 ) rt(1,4,n)=rt(1,4,n)+0.5
                         rt(2,4,n) = -d(2,1)
                         rt(3,4,n) = d(3,1)
                     endif
                  endif
               else
c
c.................it is not a hexagonal or trigonal group
c.................rotation about the b-axis
c
                  call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,-1)
                  rt(1,4,n) = d(1,2)
                  rt(3,4,n) = d(3,2)
                  if ( l(2,m).eq.12 ) rt(2,4,n)=0.5
               endif
            else if (m.eq.4) then
c
               if ( l(1,2).ge.14 .or. l(1,3).eq.14) then
                  if ( l(1,2).eq.15 ) then
c
c.....................2-axis along to (11-20) trig  and (110) tetrag
c.....................used for the p 3n21 groups
c
                      call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,-1)
                      rt(1,4,n) = d(2,1)
                      if ( l(2,m).eq.12 ) rt(1,4,n)=rt(1,4,n)+0.5
                      rt(2,4,n) = -d(2,1)
                      rt(3,4,n) = d(3,1)
                  else
ccc                  if ( l(1,2).eq.17.and.l(1,3).ne.12 )
c
c.....................2-axis normal to (10-10)
c
                     call sgrmat(rt(1,1,n),1,0,0,1,-1,0,0,0,-1)
                  endif
               else
                  call sgrmat(rt(1,1,n),-1,0,0,0,-1,0,0,0,1)
                  rt(1,4,n) = d(1,3)
                  rt(2,4,n) = d(2,3)
                  if ( iabs(l(2,m)-13).eq.1 ) rt(3,4,n) = 0.5
                  if (l(2,m).eq.16) rt(3,4,n) = 0.5
               endif
            endif
         else if (i.eq.9) then
            if (m.eq.2 .or. m.eq.3) then
               if (m.eq.2 .and. laue.gt.7) then
                  call sgrmat(rt(1,1,n),0,-1,0,1,-1,0,0,0,1)
                  if ( l(2,m).eq.12 ) rt(3,4,n)=0.33333333
                  if ( l(2,m).eq.13 ) rt(3,4,n)=0.66666667
                  if ( l(2,2).eq.3 ) ncent=1
               else
c
c.................1 fold rotation
c
                  if ( l(2,m).eq.3 ) then
c
c.....................we have a center of symmetry
c
                      ncent = 1
                   endif
c
c..................Loop exit
c
                   goto 20
               endif
            else if (m.eq.4) then
               ier=25
               if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
               return
            endif
         else if (i.eq.10 ) then
c
c...........four fold axis
c
            if ( m.ne.2 ) then
               ier=12
               if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
               return
            endif
            if ( l(2,2).eq. 3 ) then
               call sgrmat(rt(1,1,n),0,1,0,-1,0,0,0,0,-1)
               rt(1,4,n) = d(1,3)
               rt(2,4,n) = d(2,3)
               rt(3,4,n) = d(3,3)
            else
               call sgrmat(rt(1,1,n),0,-1,0,1,0,0,0,0,1)
               rt(1,4,n) = d(1,3)
               rt(2,4,n) = d(2,3)
               if ( l(2,2).eq.12 ) rt(3,4,n) = 0.25
               if ( l(2,2).eq.13 ) rt(3,4,n) = 0.5
               if ( l(2,2).eq.14 ) rt(3,4,n) = 0.75
            endif
         else if (i.eq.11) then
            ier=25
            if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
            return
         else if (i.eq.12) then
c
c...........6-axis
c
            if ( m.ne.2 )then
               ier=13
               if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
               return
            endif
            if ( l(2,2).eq.3 ) then
               call sgrmat(rt(1,1,n),-1,1,0,-1,0,0,0,0,-1)
               if ( l(1,3).eq.2.or.l(1,4).eq.2 ) rt(3,4,n)=0.5
            else
               call sgrmat(rt(1,1,n),1,-1,0,1,0,0,0,0,1)
               if ( l(2,2).gt.11.and.l(2,2).lt.18 )
     &            rt(3,4,n)=(l(2,2)-11)/6.0
            endif
c
c        else if (i.eq.13 .or. i.eq.14 .or. i.eq.15) then
c
         endif
c
         rt(1,4,n) = amod(rt(1,4,n)+5.0,1.0)
         rt(2,4,n) = amod(rt(2,4,n)+5.0,1.0)
         rt(3,4,n) = amod(rt(3,4,n)+5.0,1.0)
         rt(5,2,n) = 1728*rt(1,4,n)+144*rt(2,4,n)+12*rt(3,4,n)
         njs=n-1
         do 69 m2=1,njs
            if (rt(5,1,m2).eq.rt(5,1,n) .or.
     &          rt(5,1,m2).eq.-rt(5,1,n))  then
               if (rt(5,1,m2).eq.-rt(5,1,n)) ncent=1
               if (rt(5,2,n).ne.rt(5,2,m2)) then
                  call sgtrcf(m,rt,n,m2,lcent,laue,ier,lptx)
                  if (ier.gt.0) ierx=ier
                  ier=0
               endif
c
c..............Loop exit
c
               goto 20
            endif
 69      continue
         n = n+1
         if ( n.gt.25 ) then
            ier=14
            if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
            return
         endif
 59      ident=0
         nxl = n-1
c
c........exit
c
         if ( nxl.lt.nxi )  goto 49
         do 195 nx=nxi,nxl
            do 196 m1=2,nx
               call sgmtml(rt,m1,rt,nx,rt,n)
               njs=n-1
               do 194 m2=1,njs
                  if (rt(5,1,m2).eq.rt(5,1,n) .or.
     &                rt(5,1,m2).eq.-rt(5,1,n))  then
                     if (rt(5,1,m2).eq.-rt(5,1,n)) ncent=1
c
c....................Loop exit
c
                     go to 196
                  endif
194            continue
               n = n+1
               if ( n.gt.25 ) then
                  ier=15
                  if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
                  return
               endif
196         continue
c
c............Loop exit
c
            if ( n-1.eq.nxl ) goto 49
195      continue
         nxi = nxl+1
c
c........Repeat whole cycle
c
         go to 59
c
c........After completing loop cycle
c
 49      continue
c
c........search for a / . a mirror perpendicular to this axis
c........Next m
c
         if ( l(1,m).lt.12 .or. l(2,m).eq.3 )  goto 20
         do 39 i=2,3
c
c...........Next m
c
            if ( l(i,m).eq.0 ) goto 20
c
c...........exit loop 39
c
            if ( l(i,m).eq.19 ) goto 29
            if ( l(i,m).lt.12 ) then
               ier=16
               if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
               return
            endif
 39      continue
c
c........Test for another matrix element
c
 29      if ( l(i+1,m).le.1 ) then
            ier=17
            if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
            return
         endif
         i = iabs(l(i+1,m)-5)
c
c........Next matrix element
c        Non ortodox control flow
c
         goto 10
c
c.....end of loop
c
 20   continue
      neqv = n-1
      do 11 i=1,3
         do 21 k=1,neqv
            do 31 j=1,3
               jrt(i,j,k) = rt(i,j,k)
 31         continue
            jrt(i,4,k) = 12*rt(i,4,k)+144.1
            jrt(i,4,k) = jrt(i,4,k)-12*(jrt(i,4,k)/12)
 21      continue
 11   continue
c
c.....print results and construct interface symmetry matrix
c
      call sgprnt(spg(1:30),lpt)
      if ( ierx.eq.0 ) return
      ier = ierx
c
c.....Error handling
c
      if (lptx.ge.0) call sgerrs(spg(1:30),ier,lptx)
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine ifcent (l, d, lcent, ier)
c
c.....ifcent - determines parameters for I-F centered lattices
c
      dimension         d(3,3),l(4,4)
c
c.....is the lattice I or F-centered?
      if ( lcent.lt.5 ) then
c
c........is the lattice C-centered?
         if ( lcent.ne.4 ) then
            ier=23
            return
         endif
c
c........c-centred.  an A normal to C
         if ( l(3,2).eq.4.or.l(4,2).eq.4 )  then
c
c............C 4*/A * *
             d(1,3) = 0.25
             d(2,3) = 0.25
             if ( l(1,4).eq.3.or.l(1,4).eq.10 ) d(1,1)=0.5
             return
         endif
c
c........if ( l(3,2).eq.0 ) d(1,1)=2.0*d(2,2)+d(1,1)
         if ( d(1,1).eq.0.0 ) d(1,1)=2.0*d(2,2)
c
c........is there a 21 on the diagonal?
         if ( l(1,4).eq.13.and.l(2,4).eq.12 )  then
c
c...........P 4N/N * *
            d(1,3) = 0.5
            return
         endif
         if ( l(2,2).le.0 ) return
c
c........is there a N-glide normal to (110)?
         if ( l(1,4).ne.10 ) return
         if ( l(2,2).gt.15 ) return
         d(1,1) = d(1,1)-2.0*d(2,2)
c
c........P 4N/N * *
         d(1,3) = 0.5
         return
      endif
c
c.....yes.
c
c.....if there is a C along (110) place the D at Y=1/4
      if ( l(1,4).eq.2 ) d(2,1)=d(2,1)+0.5
c
c.....is this I 41/A * * or F 41/D * * ?
      if ( l(4,2).ne.4.and.l(4,2).ne.11 ) then
c
c........is there a 41 present?
         if ( l(2,2).ne.12 ) return
c
c........yes.
c........F-centered?
         if ( lcent.eq.6 ) then
c
c...........F 41 * *
            d(1,3) = 0.25
            d(2,3) = 0.75
            return
         endif
         d(2,3) = 0.5
c
c........set the B-axis translation flags for I 41 2 2
         if ( lcent.eq.5 ) d(1,2) = 0.5
         d(3,2) = 0.75
         return
      endif
c
c.....yes
c
      d(1,3) = 0.25
      if ( lcent.eq.5 ) d(2,3) = 0.75
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine ortho (d, l, laue, lcent, ier, id)
c
      dimension         d(3,3),l(4,4)
c
c.....it is orthorhombic
      laue = 3
c
c.....set up counts of the various types of mirrors.
      im = 0
      ir = 0
      ia = 0
      ib = 0
      ic = 0
      id = 0
      i21 = 0
c
c.....do we have a 2-axis along A
      if ( l(1,2).ne.13 ) then
c
         ir = 1
         if ( l(1,2).eq.9 ) im=4
c        if ( l(1,2).eq.4 ) ier =5  return
         if ( l(1,2).eq.3 ) ib=1
         if ( l(1,2).eq.2 ) ic=1
         if ( l(1,2).eq.11 ) id=1
         if ( l(1,3).eq.4.or.l(1,3).eq.10 ) d(1,1)=0.5
         if ( l(1,4).eq.4.or.l(1,4).eq.10 ) d(1,1)=d(1,1)+0.5
c
c.....yes is it a 21?
      else if(l(2,2).eq.12) then
         d(1,2) = 0.5
         d(1,3) = 0.5
         i21 = 4
      endif
c
c.....do we have a 2-axis along B
      if ( l(1,3).ne.13 ) then
c
         ir = ir+1
         if ( l(1,3).eq.9 ) im=im+2
         if ( l(1,3).eq.4 ) ia=1
C        if ( l(1,3).eq.3 ) ier =5 return
         if ( l(1,3).eq.2 ) ic=ic+1
         if ( l(1,3).eq.11 ) id=id+1
         if ( l(1,2).eq.3.or.l(1,2).eq.10 ) d(2,2)=0.5
         if ( l(1,4).eq.3.or.l(1,4).eq.10 ) d(2,2)=d(2,2)+0.5
c
c.....yes, is it a 21?
      else if (l(2,3).eq.12) then
         d(2,1) = 0.5
         d(2,3) = 0.5
         i21 = i21+2
      endif
c
c.....do we have a 2-axis along C
      if ( l(1,4).ne.13 ) then
         ir = ir+1
         if ( l(1,4).eq.9 ) im=im+1
         if ( l(1,4).eq.4 ) ia=ia+1
         if ( l(1,4).eq.3 ) ib=ib+1
C        if ( l(1,4).eq.2 ) ier=5 return
         if ( l(1,4).eq.11 ) id=id+1
         if ( l(1,2).eq.2.or.l(1,2).eq.10 ) d(3,3)=0.5
         if ( l(1,3).eq.2.or.l(1,3).eq.10 ) d(3,3)=d(3,3)+0.5
c
c.....yes , is it a 21
      else if (l(2,4).eq.12) then
         d(3,1) = 0.5
         d(3,2) = 0.5
         i21 = i21+1
      endif
c.....if there are 3 mirrors check for centering
c     which may alter the origin location
c
      if ( ir.eq.3 )  then
c........3 mirrors present.  is the lattice centered?
         if ( lcent.eq.1 ) return
c........yes . is it A-centered?
         if ( lcent.eq.2 ) then
c...........an A-centered lattice.
c           only one B or C glide present.relocate the mirrors by A
            if ( ib+ic.ne.1 ) return
            if ( ia.eq.2 ) return
            d(2,2) = d(2,2)+0.5
            d(3,3) = d(3,3)+0.5
            return
         endif
c
c........no.   is it B-centered?
         if ( lcent.eq.3 ) then
c
c...........a B-centered lattice
            if ( ia+ic.ne.1 ) return
            if ( ib.eq.2 ) return
            d(1,1) = d(1,1)+0.5
            d(3,3) = d(3,3)+0.5
            return
         endif
c
c........no.  is it C-centered?
         if ( lcent.eq.4 ) then
c
c...........a C-centered lattice
            if ( ia+ib.ne.1 ) return
            if ( ic.eq.2 ) return
            d(1,1) = d(1,1)+0.5
            d(2,2) = d(2,2)+0.5
            return
         endif
c
c........no.   is it I-centered?
         if ( lcent.ne.5 ) return
c
c........yes.  if only 1 glide plane shift the mirrors by I
         if ( ia+ib+ic.ne.1 ) return
         d(1,1) = d(1,1)+0.5
         d(2,2) = d(2,2)+0.5
         d(3,3) = d(3,3)+0.5
         return
      endif
c
c.....less than 3 mirrors. set up the 2-axes locations
      if ( i21.eq.4.or.i21.eq.5.or.i21.eq.7 ) d(1,2)=0.0
      if ( i21.eq.6.or.i21.eq.7 ) d(1,3)=0.0
      if ( i21.eq.3 ) d(2,1)=0.0
      if ( i21.eq.2.or.i21.eq.6.or.i21.eq.7 ) d(2,3)=0.0
      if ( i21.eq.1.or.i21.eq.3.or.i21.eq.7 ) d(3,1)=0.0
      if ( i21.eq.5 ) d(3,2)=0.0
      if ( im.le.0 ) return
      if ( im.eq.1.and.(i21.eq.4.or.i21.eq.2)) then
         if ( d(3,3).eq.0.0 ) return
         d(3,3)=0.0
         d(3,2) = d(3,2)+0.5
         return
      endif
      if ( im.eq.2.and.(i21.eq.4.or.i21.eq.1)) then
         if ( d(2,2).eq.0.0 ) return
         d(2,2)=0.0
         d(2,1) = d(2,1)+0.5
         return
      endif
      if ( im.eq.4.and.(i21.eq.2.or.i21.eq.1)) then
         if ( d(1,1).eq.0.0 ) return
         d(1,1)=0.0
         d(1,3) = d(1,3)+0.5
         return
      endif
c     if ( im.eq.3.amd.i21.eq.4 ) ier=5 return
c     if ( im.eq.5.and.i21.eq.2 ) ier=5 return
c     if ( im.eq.6.and.i21.eq.1 ) ier=5 return
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine sgerrs (sgp,ierr,lpt)
c
c.....sgerrs - writes out symmetry errors.
c
      character*(30)    sgp
c
      ier=ierr+1
      write (lpt,999) ier,sgp
c
c.....gigant if
c
      if (ier.eq.1) then
         write (lpt,1)
      else if (ier.eq.2) then
         write (lpt,2)
      else if (ier.eq.3) then
         write (lpt,3)
      else if (ier.eq.4) then
         write (lpt,4)
      else if (ier.eq.5) then
         write (lpt,5)
      else if (ier.eq.6) then
         write (lpt,6)
      else if (ier.eq.7) then
         write (lpt,7)
      else if (ier.eq.8) then
         write (lpt,8)
      else if (ier.eq.9) then
         write (lpt,9)
      else if (ier.eq.10) then
         write (lpt,10)
      else if (ier.eq.11) then
         write (lpt,11)
      else if (ier.eq.12) then
         write (lpt,12)
      else if (ier.eq.13) then
         write (lpt,13)
      else if (ier.eq.14) then
         write (lpt,14)
      else if (ier.eq.15) then
         write (lpt,15)
      else if (ier.eq.16) then
         write (lpt,16)
      else if (ier.eq.17) then
         write (lpt,17)
      else if (ier.eq.18) then
         write (lpt,18)
      else if (ier.eq.19) then
         write (lpt,19)
      else if (ier.eq.20) then
         write (lpt,20)
      else if (ier.eq.21) then
         write (lpt,21)
      else if (ier.eq.22) then
         write (lpt,22)
      else if (ier.eq.23) then
         write (lpt,23)
      else if (ier.eq.24) then
         write (lpt,24)
      else if (ier.eq.25) then
         write (lpt,25)
      else if (ier.eq.26) then
         write (lpt,26)
      endif
c
      return
c
c.....formats
c
    1 format ('either a 5-axis anywhere or a 3-axis in field 4')
    2 format ('less than 2 operator fields were found')
    3 format ('lattice operator was not a p, a, b, c, i, f or r')
    4 format ('rhombohedral lattice without a 3-axis')
    5 format ('minus sign does not preceed 1, 2, 3, 4 or 6')
    6 format ('lattice s. r. found an error')
    7 format ('1st operator in a field was a space. impossible')
    8 format ('i for computed go to out of range.')
    9 format ('an a-glide mirror normal to a')
   10 format ('a b-glide mirror normal to b')
   11 format ('a c-glide mirror normal to c')
   12 format ('d-glide in a primitive lattice')
   13 format ('a 4-axis not in the 2nd operator field')
   14 format ('a 6-axis not in the 2nd operator field')
   15 format ('more than 24 matrices needed to define the group')
   16 format ('more than 24 matrices needed to define the group')
   17 format ('improper construction of a rotation operator')
   18 format ('no mirror following a /')
   19 format ('a translation conflict between operators')
   20 format ('the 2bar operator is not allowed')
   21 format ('3 fields are legal only in r-lattices and m3 cubic')
   22 format ('syntax error. expected i-43d at this point.')
   23 format ('error unknown')
   24 format ('a or b centered tetragonal?  impossible!!!!')
   25 format ('only one bar allowed')
   26 format ('either a 5-axis anywhere or a 3-axis in field 4')
c
  999 format ('error no. ',i3,' in processing space group symbol ',a30)
c
      end
c
c-----------------------------------------------------------------------
c
      subroutine sglatc (k, l, d, lcent, laue, naxis, ier, ncubic, id)
c
      dimension         d(3,3), l(4,4)
c
c.....sglatc - determine the laue group and unique axix if monoclinic
c
      if (k .lt.3 ) then
         if(l(1,2).eq.17) then
c
c...........6/m
c
            laue=11
            return
         else if(l(1,2).eq.14) then
c
c...........2bar
c
            laue=8
            return
         else if(l(1,2).eq.12) then
c
c...........1bar
c
            laue=1
            return
         else if(l(1,2).ne.15) then
c
c...........2/m, B-axis unique
c
            im=2
            laue = 2
            naxis = 2
            ia = 4
            ic = 2
            na = 1
            nb = 2
            nc = 3
            if ( l(2,im).eq.12 ) d(nb,naxis)=0.5
            if ( l(3,im).eq.ia.or.l(3,im).eq.10 ) d(na,naxis)=0.5
            if ( l(3,im).eq.ic.or.l(3,im).eq.10 ) d(nc,naxis)=0.5
            if ( l(4,im).eq.ia.or.l(4,im).eq.10 ) d(na,naxis)=0.5
            if ( l(4,im).eq.ic.or.l(4,im).eq.10 ) d(nc,naxis)=0.5
            return
         else
c
c...........4/m
c
            laue=4
c
c...........is there a n-glide normal to c?
c
            if ( lcent.le.3 ) then
               if (l(3,2).eq.10) then
c
c.................P 4N/N * *
c
                  d(1,3) = 0.5
                  return
                endif
                if (l(4,2).eq.10) d(2,3)=0.5
                return
c
c...........is it c-centered?
c
            else if (lcent.eq.4) then
c
c..............if there is no A-glide normal to C we are through
c
               if ( l(3,2).ne.4.and.l(4,2).ne.4 ) return
               d(1,3) = 0.25
               d(2,3) = 0.25
               if ( l(4,2).eq.4 ) d(2,3)=0.75
               return
c
c...........is it I-centered or F-centered
c
            else if (lcent.ge.5) then
c
c..............is there an A-glide or a D-glide normal to C?
c
               if ( l(4,2).ne.4.and.l(4,2).ne.11 ) then
c
c.................no
c
                  call ifcent (l,d,lcent,ier)
                  return
               else
c
c.................yes
c
                  d(1,3) = 0.75
                  if ( lcent.eq.5 ) d(2,3) = 0.25
                  return
               endif
            endif
         endif
c
      else if (k .eq. 3) then
c
c........3 fields.  M3 cubic. (R3R has been taken care of)
c
         if ( l(1,3).ne.14 ) then
            ier=20
            return
         endif
         laue = 13
c
c........set the B-axix translation flag if A 21 along A
c
         if ( l(2,2).eq.12 ) d(2,1)=0.5
c
c........set the C-axis translation flag if an A-glide normal to C
c
         if ( l(1,2).eq.3.or.l(1,2).eq.4 ) d(3,3)=0.5
         ncubic=1
         return
c
      else if (k .gt. 3) then
c
c.........four fields read.
c
          if ( l(1,3).eq.14 ) then
c
c...........it is m3m cubic
c
            laue = 14
c
c...........set the C-axis translation flag if an A-glide normal to C
c
            if ( l(1,2).eq.3.or.l(1,2).eq.4 ) d(3,3)=0.5
c
c...........is a 4N-axis specified?
c
            if ( l(1,2).ne.15 ) then
               ncubic=1
               return
            endif
c
c.......... yes.  is it 4bar?
c
            if ( l(2,2).eq.3 ) then
c
c..............4B.  is it 4B 3 M?
c
              if ( l(1,4).eq.9 )  then
                 ncubic=1
                 return
              endif
c
c.............no.  is it 4B 3 D?
c
              if ( l(1,4).eq.11 ) then
c
c................I 4B 3 D we hope
c
                 if ( lcent.ne.5 ) then
                    ier=21
                    return
                 endif
                 d(1,3) = 0.75
                 d(2,3) = 0.25
                 d(3,3) = 0.75
                 ncubic=1
                 return
              endif
c
c.............no.
c
              d(1,3) = 0.5
              d(2,3) = 0.5
              d(3,3) = 0.5
              ncubic=1
              return
           endif
c
c..........no.  is it a 4?
c
           if ( l(2,2).lt.12 .or. l(2,2).gt.14 ) then
              ncubic=1
              return
           endif
c
c..........no. is it a 41?
c
           if ( l(2,2).eq.12 ) then
c
c.............41-axis. is it F 41 3 2?
c
              if ( lcent.eq.6 ) then
c
c................F 41 3 2
c
                 d(1,3) = 0.25
                 d(2,3) = 0.25
                 ncubic=1
                 return
              endif
c
c.............no. it is either P 41 3 2 or I 41 3 2
c
              d(1,3) = 0.25
              d(2,3) = -0.25
              ncubic=1
              return
           endif
c
c..........no . is it a 42?
c
           if ( l(2,2).eq.13 ) then
c
c.............P 42 3 2
c
              d(1,3) = 0.5
              d(2,3) = 0.5
              ncubic=1
              return
           endif
c
c..........no.  it must be a 43
c
c..........P 43 3 2
c
           if ( lcent.eq.6 ) then
c
c.............F 41 3 2
c
              d(1,3) = 0.25
              d(2,3) = 0.25
              ncubic=1
              return
           endif
           d(1,3) = 0.75
           d(2,3) = 0.25
           ncubic=1
           return
         else if (l(1,2).eq.17) then
            if ( l(1,3).eq.12 ) then
c
c..............we have something like P 6N 1 *
c
               if ( l(1,4).ne.12 ) then
c
c.................it is hexagonal 6/mmm
c
                  laue = 12
                  return
               endif
c
c..............6/M
c
               laue = 11
               return
            endif
c
c...........it is hexagonal 6/mmm
c
            laue = 12
            return
c
         else if (l(1,2).eq.14) then
c
c...........it is trigonal P3**
c
            if ( l(1,3).eq.12 ) then
c
c..............it is trigonal 31*
c
               if ( l(1,4).eq.12 ) then
c
c.................3BAR
c
                  laue = 8
                  return
               endif
c
c..............it is trigonal 31M
c
               laue = 10
               return
            endif
            if ( l(1,4).ne.12 ) then
c
c..............it is hexagonal 6/mmm
c
               laue = 12
               return
            endif
c
c...........it is trigonal 3M1
c
            laue = 9
            return
         else if (l(1,2).eq.15) then
c
c...........it is tetragonal 4/mmm
c
            laue = 5
c
c...........if there is an N-glide normal to C place any
c
c           mirror normal to A at 1/4
            if ( l(3,2).eq.10.or.l(4,2).eq.10 ) d(1,1)=0.5
c
c...........if there is an A-glide normal to C place any
c
c           mirror normal to (110) at 1/4
            if ( l(3,2).eq.4.or.l(4,2).eq.4 ) d(2,2)=0.25
c
c...........if there is a 21 along B move place it at X=1/4
c
            if ( l(1,3).eq.13.and.l(2,3).eq.12 ) d(1,2)=0.5
c
c...........if there is a 21 along (110) move place it at X=1/4
c
            if ( l(1,4).eq.13.and.l(2,4).eq.12 ) d(2,1)=0.5
c
c...........if there is either a B- or N-glide normal to the A-axis
c           shift the mirror by 1/4 along the A-axis
c
            if ( l(1,3).eq.3.or.l(1,3).eq.10 ) d(1,1)=d(1,1)+0.5
c
c...........if there is either a B- OR N-glide normal to (110)
c           shift the mirror by 1/4 along the A-axis
c
            if ( l(1,4).eq.3.or.l(1,4).eq.10 ) d(2,2)=d(2,2)+0.25
c
c...........set the Z location for 2-axes along (110)
c
            if ( l(2,2).gt.11.and.l(2,2).lt.15.and.l(2,3).ne.12 )
     &      d(3,1)=-(l(2,2)-11)/4.0
c
c...........set the Z-translation for 21-axes along (110)
c
            if ( l(1,4).eq.13.and.l(2,4).ne.12 ) then
            else
             if ( l(2,2).gt.11.and.l(2,2).lt.15 )
     &            d(3,1)=(l(2,2)-11)/4.0
            endif
c
c...........set the Z-translation for 21-axes along B
c
            if ( l(1,3).eq.13.and.l(2,3).ne.12 ) then
            else
            if ( l(2,2).gt.11.and.l(2,2).lt.15 )
     &           d(3,2)=(l(2,2)-11)/4.0
            endif
c
c...........place the D in F 4* D * at Y=7/8
c
            if ( l(1,3)+l(3,2).eq.11.and.lcent.eq.6 ) d(2,1)=0.75
c
c...........M in F 4** * * at X=1/8 if there is a C along (110)
c
            if ( l(1,4).eq.2.and.lcent.eq.6 ) d(1,1)=0.5
c
c...........is this a 4bar?
c
            if ( l(2,2).eq.3 ) then
c
c..............translations due to diagonal symmetry operation
c
               if ( l(1,3).eq.11.and.lcent.eq.6 ) d(3,1)=0.25
c
c..............if * 4B * 21 we want the mirror at X=1/4
c
               if ( l(1,4).eq.13.and.l(2,4).eq.12 ) d(1,1)=0.5
c
c..............C- or a N-glide along (110) set the 2-axis at Z=1/4
c
               if ( l(1,4).eq.2.or.l(1,4).eq.10 ) d(3,2)=0.5
c
c..............a B- or a N-glide along (110) set the 2 at X=1/4
c
               if ( l(1,4).eq.3.or.l(1,4).eq.10 ) d(1,2)=0.5
               if ( l(1,4).ne.11 ) return
               if ( lcent.eq.5 ) d(1,2) = 0.5
               d(3,2) = 0.75
               return
            endif
c
c...........is the lattice primitive?
c
            if ( lcent.gt.1 ) then
               call ifcent  (l,d,lcent,ier)
               return
            endif
c
c...........yes.  do we have a N-glide normal to C?
c
            if ( l(3,2).eq.10.or.l(4,2).eq.10 ) then
c
c..............P 4N/N * *
c
               d(1,3) = 0.5
               return
            endif
c
c...........no.  do we have a 21 along B?
c
            if ( l(1,3).eq.13.and.l(2,3).eq.12 ) then
               d(1,3) = 0.5
               d(2,3) = 0.5
               return
            endif
c
c...........no. do we have a N-glide normal to A?
c
            if ( l(1,3).ne.10 ) return
            if ( l(2,2).le.0 )  return
            if ( l(2,2).gt.15)  return
            d(1,3) = 0.5
            d(2,3) = 0.5
            return
         else if (l(1,2).eq.12) then
            if ( l(1,3).eq.12 ) then
               if ( l(1,4).eq.12 ) then
c
c.................1BAR
c
                  laue = 1
                  return
               endif
c
c..............it is C-axis unique monoclinic
c
               laue = 2
               naxis = 3
               ia = 4
               ic = 3
               na = 1
               nb = 3
               nc = 2
               im = 4
            else
c
c..............it is not C-axis unique monoclinic
c
               if ( l(1,4).ne.12 ) then
                  call ortho (d, l, laue, lcent, ier, id)
                  return
               endif
               im = 3
c
c..............it is B-axis unique monoclinic. (FULL SYMBOL USED)
c
               laue = 2
               naxis = 2
               ia = 4
               ic = 2
               na = 1
               nb = 2
               nc = 3
            endif
            if ( l(2,im).eq.12 ) d(nb,naxis)=0.5
            if ( l(3,im).eq.ia.or.l(3,im).eq.10 ) d(na,naxis)=0.5
            if ( l(3,im).eq.ic.or.l(3,im).eq.10 ) d(nc,naxis)=0.5
            if ( l(4,im).eq.ia.or.l(4,im).eq.10 ) d(na,naxis)=0.5
            if ( l(4,im).eq.ic.or.l(4,im).eq.10 ) d(nc,naxis)=0.5
            return
         else if (l(1,3).eq.12) then
            if ( l(1,4).ne.12 ) then
               call ortho (d, l, laue, lcent, ier, id)
               return
            endif
c
c...........it is A-axis unique monoclinic
c
            laue = 2
            naxis = 1
            ia = 3
            ic = 2
            na = 2
            nb = 1
            nc = 3
            im = 2
            if ( l(2,im).eq.12 ) d(nb,naxis)=0.5
            if ( l(3,im).eq.ia.or.l(3,im).eq.10 ) d(na,naxis)=0.5
            if ( l(3,im).eq.ic.or.l(3,im).eq.10 ) d(nc,naxis)=0.5
            if ( l(4,im).eq.ia.or.l(4,im).eq.10 ) d(na,naxis)=0.5
            if ( l(4,im).eq.ic.or.l(4,im).eq.10 ) d(nc,naxis)=0.5
            return
         else
c
c...........It may be orthorhombic
c
            call ortho (d, l, laue, lcent, ier, id)
            return
         endif
      endif
c
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine sglpak (l, ier)
c
      dimension         l(4)
c
      if ( l(2).lt.12 ) ier=4
      if ( l(2).gt.17 ) ier=4
      l(1) = l(2)
      l(2) = 3
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine sgmtml (x, i, y, j, z, k)
c
      dimension         x(5,4,25), y(5,4,25), z(5,4,25)
c
      do 100 l=1,4
         do 101 m=1,4
            z(l,m,k) = 0.0
            do 102 n=1,4
               z(l,m,k) = z(l,m,k)+y(l,n,j)*x(n,m,i)
102         continue
101      continue
100   continue
      z(1,4,k) = amod(5.0+z(1,4,k),1.0)
      z(2,4,k) = amod(5.0+z(2,4,k),1.0)
      z(3,4,k) = amod(5.0+z(3,4,k),1.0)
      z(5,1,k) = 81*(2*z(1,1,k)+3*z(1,2,k)+4*z(1,3,k))
     &             +9*(2*z(2,1,k)+3*z(2,2,k)+4*z(2,3,k))
     &                +2*z(3,1,k)+3*z(3,2,k)+4*z(3,3,k)
      z(5,2,k) = 1728*z(1,4,k)+144*z(2,4,k)+12*z(3,4,k)
      z(5,3,k) = 0.0
      z(5,4,k) = 0.0
c
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine        sgprnt (spg, lpt)
c
c.....sgprnt - prints out spatial symmetry information
c
      character*(*)     spg
      integer           lpt
      include           'spgcom.inc'
c
      character*(1)     nax(3)
      character*(3)     outl(3,2,24)
      character*(4)     xyz(12), polar(8), tra(11)
      character*(8)     chlaue(14)
      character*(12)    ltyp(7), syst(8)
c
      dimension         ncvt(7),cenv(3,6)
      dimension         nsys(14)
      dimension         ig(3,4),ncen2(3,4),ncenv2(3,6)
      dimension         tra2(11)
c
c.....datas
c
      data
     & tra2( 1),tra2( 2),tra2( 3) /0.0      ,0.0      ,0.1666667/,
     & tra2( 4),tra2( 5),tra2( 6) /0.25     ,0.3333333,0.0      /,
     & tra2( 7),tra2( 8),tra2( 9) /0.5      ,0.0      ,0.6666667/,
     & tra2(10),tra2(11)          /0.75     ,0.8333333/
c
      data
     & ncenv2(1,1),ncenv2(2,1),ncenv2(3,1) /0,6,6/,
     & ncenv2(1,2),ncenv2(2,2),ncenv2(3,2) /6,0,6/,
     & ncenv2(1,3),ncenv2(2,3),ncenv2(3,3) /6,6,0/,
     & ncenv2(1,4),ncenv2(2,4),ncenv2(3,4) /6,6,6/,
     & ncenv2(1,5),ncenv2(2,5),ncenv2(3,5) /4,8,8/,
     & ncenv2(1,6),ncenv2(2,6),ncenv2(3,6) /8,4,4/
c
      data ltyp /   'primitive   ', 'A-centered  ', 'B-centered  ',
     &              'C-centered  ', 'I-centered  ', 'F-centered  ',
     &              'R-centered  ' /
      data syst /   'triclinic   ', 'monoclinic  ', 'orthorhombic',
     &              'tetragonal  ', 'trigonal-R  ', 'trigonal-P  ',
     &              'hexagonal   ', 'cubic       ' /
c
      data chlaue / '1bar    ', '2/m     ', 'mmm     ', '4/m     ',
     &              '4/mmm   ', '3bar    ', '3barm   ', '3bar    ',
     &              '3barm1  ', '3bar1m  ', '6/m     ', '6/mmm   ',
     &              'm3      ', 'm3m     ' /
c
      data polar  / 'x   ',   'y   ',   'x y ',   'z   ',   'x z ',
     &              'y z ',   'xyz ',   '111 ' /
c
      data nax    / 'a',      'b',      'c' /
c
      data xyz   /  '-z  ',   '-y  ',   '-x  ',   'x-y ',   'err ',
     &              'y-x ',   ' x  ',   ' y  ',   ' z  ',   '+x  ',
     &              '+y  ',   '+z  ' /
c
      data tra   /  '    ',   'err ',   '1/6 ',   '1/4 ',   '1/3 ',
     &              'err ',   '1/2 ',   'err ',   '2/3 ',   '3/4 ',
     &              '5/6 ' /
c
      data
     & nsys( 1),nsys( 2),nsys( 3),nsys( 4),nsys( 5) /1,2,3,4,4/,
     & nsys( 6),nsys( 7),nsys( 8),nsys( 9),nsys(10) /5,5,6,6,6/,
     & nsys(11),nsys(12),nsys(13),nsys(14)          /7,7,8,8/
c
      data
     & ncvt(1),ncvt(2),ncvt(3),ncvt(4),ncvt(5) /1,2,2,2,2/,
     & ncvt(6),ncvt(7)                         /4,3/
c
      data
     & cenv(1,1),cenv(2,1),cenv(3,1) /0.0      ,0.5      ,0.5      /,
     & cenv(1,2),cenv(2,2),cenv(3,2) /0.5      ,0.0      ,0.5      /,
     & cenv(1,3),cenv(2,3),cenv(3,3) /0.5      ,0.5      ,0.0      /,
     & cenv(1,4),cenv(2,4),cenv(3,4) /0.5      ,0.5      ,0.5      /,
     & cenv(1,5),cenv(2,5),cenv(3,5) /0.3333333,0.6666667,0.6666667/
     & cenv(1,6),cenv(2,6),cenv(3,6) /0.6666667,0.3333333,0.3333333/
c
      ncv = ncvt(lcent)
      mult = ncv*neqv*(ncent+1)
      lsys = nsys(laue)
      cen(1,1) = 0.0
      cen(2,1) = 0.0
      cen(3,1) = 0.0
      ncen2(1,1)=0
      ncen2(2,1)=0
      ncen2(3,1)=0
      if ( ncv.gt.1 )  then
         j = lcent-1
         if ( lcent.eq.6 ) j=1
         if ( lcent.eq.7 ) j=5
         do 100 i=2,ncv
            cen(1,i) = cenv(1,j)
            cen(2,i) = cenv(2,j)
            cen(3,i) = cenv(3,j)
            ncen2(1,i)=ncenv2(1,j)
            ncen2(2,i)=ncenv2(2,j)
            ncen2(3,i)=ncenv2(3,j)
            j = j+1
100      continue
      endif
      npx = 1
      npy = 2
      npz = 4
      npxyz = 0
      npyxz = 1
      do 120 i=1,neqv
         if ( jrt(1,1,i).le.0 ) npx=0
         if ( jrt(2,2,i).le.0 ) npy=0
         if ( jrt(3,3,i).le.0 ) npz=0
         if ( jrt(1,3,i).gt.0 ) npxyz=8
         if ( jrt(1,3,i).lt.0 ) npyxz=0
120   continue
      npol = (npx+npy+npz+npxyz*npyxz)*(1-ncent)
      naxi=naxis
      if(lsys.eq.4.or.lsys.eq.6.or.lsys.eq.7) naxi=3
      neqv2=neqv+2
c
c.....Creamos la matriz rotm
c
      do 211 i=1,neqv
         do 212 j=1,3
            do 213 k=1,3
               rotm(j,k,i)=jrt(j,k,i)
 213        continue
            rotm(j,4,i)=tra2(jrt(j,4,i)+1)
 212     continue
 211  continue
c
c.....Write the results
c
      if (lpt .ge. 0) then
         write (lpt,1005) spg
         if (ncent.eq.0) write (lpt,1006)
         if (ncent.eq.1) write (lpt,1007)
         write (lpt,1008) ltyp(lcent)
         write (lpt,1009) syst(lsys)
         write (lpt,1010) chlaue(laue)
         write (lpt,1011) mult
         if (naxi.gt.0) write (lpt,1012) nax(naxi)
         if (naxi.eq.0) write (lpt,1013)
         if (npol.gt.0) write (lpt,1014) polar(npol)
         if (npol.le.0) write (lpt,1015)
         write (lpt,1016) neqv
      endif
c
c.....redefine npol
c
      npol=npol*10000+lsys*1000+mult
c
c.....first part concluded, start with symmetry relations
c
      do 200 i=1,neqv
         do 190 j=1,3
            ij = 2*jrt(j,1,i)+3*jrt(j,2,i)+4*jrt(j,3,i)+5
            ik = jrt(j,4,i)+1
            if (ik.gt.1 .and. ij.gt.6) ij=ij+3
            outl(j,2,i) = xyz(ij)
            outl(j,1,i) = tra(ik)
 190     continue
 200  continue
      if (lpt .ge. 0) then
         write (lpt,1026) ((outl(i1,1,i),outl(i1,2,i),i1=1,3),i=1,neqv)
c
c........write the symmetry matrices and lattice centering vectors.
c
         write (lpt, 7001) neqv
         write (lpt, 7020) ncv
         do 7030 i=1,ncv
            write (lpt, 7025) (cen(j,i), j=1,3)
 7030    continue
      endif
c
c.....Formats.
c
 1026 format (3(2x,'(',2(2a3,1h,),2a3,') '))
 1005 format (1x,'the space group is ',a30)
 1006 format (1x,'the structure is noncentrosymmetric')
 1007 format (1x,'the structure is centrosymmetric')
 1008 format (1x,'the lattice is ',a12)
 1009 format (1x,'the system is ',a12)
 1010 format (1x,'the laue group is ',a8)
 1011 format (1x,'the multiplicity of the general position is ',i3)
 1012 format (1x,'the unique axis is ',a1)
 1013 format (1x,'there is no unique axis')
 1014 format (1x,'the space group is polar along ',a4/)
 1015 format (1x,'the space group is not polar'/)
 1016 format (1x,'the number of generated symmetry relations is ',i2)
 7001 format (//1x,'the number of Symmetry Matrices is',i5)
 7020 format (1x,'the number of Lattice Centering Vectors is ',i5)
 7025 format (1x,'cenvec : ', 3f10.7)
c
      return
      end
c
c----------------------------------------------------------------------
c
      subroutine        sgrmat (rt, a, b, c, d, e, f, g, h, o)
c
c.....sgrmat - builds the rt(,) matrix out of the a..o rotational
c              elements.
c              rt(,) is a 5x4 matrix, which contains a 3x3 rotation
c              plus a column for the traslation, a row that could
c              represent a deformation of the axes (set to 0), and ...
c
      integer           a, b, c, d, e, f, g, h, o
c
      dimension         rt(5,4)
c
      rt(1,1) = a
      rt(1,2) = b
      rt(1,3) = c
      rt(1,4) = 0.0
      rt(2,1) = d
      rt(2,2) = e
      rt(2,3) = f
      rt(2,4) = 0.0
      rt(3,1) = g
      rt(3,2) = h
      rt(3,3) = o
      rt(3,4) = 0.0
      rt(4,1) = 0.0
      rt(4,2) = 0.0
      rt(4,3) = 0.0
      rt(4,4) = 1.0
      rt(5,1) = 81 * (2 * rt(1,1) + 3 * rt(1,2) + 4 * rt(1,3))
     &         + 9 * (2 * rt(2,1) + 3 * rt(2,2) + 4 * rt(2,3))
     &         + 2 * rt(3,1) + 3 * rt(3,2) + 4 * rt(3,3)
      rt(5,2) = 0.0
      rt(5,3) = 10.0
      rt(5,4) = 20.
c
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine        sgtrcf (m, rt, n, m2, lcent, laue, ier, lpt)
c
      dimension         rt(5,4,25)
      dimension         icenv(3,5),ncvt(7),jcvt(7)
c
      data
     & icenv(1,1),icenv(2,1),icenv(3,1) /0,0,0/,
     & icenv(1,2),icenv(2,2),icenv(3,2) /0,6,6/,
     & icenv(1,3),icenv(2,3),icenv(3,3) /6,0,6/,
     & icenv(1,4),icenv(2,4),icenv(3,4) /6,6,0/,
     & icenv(1,5),icenv(2,5),icenv(3,5) /6,6,6/
c
      data
     & ncvt(1),ncvt(2),ncvt(3),ncvt(4),ncvt(5) /1,2,3,4,5/,
     & ncvt(6),ncvt(7)                         /4,1/
c
      data
     & jcvt(1),jcvt(2),jcvt(3),jcvt(4),jcvt(5) /1,1,2,3,4/,
     & jcvt(6),jcvt(7)                         /1,1/
c
      ier = 0
      irn = rt(5,2,n)
      irm = rt(5,2,m2)
      irx = mod((irn/144+irm/144),12)
      iry = mod((irn/12 +irm/12 ),12)
      irz = mod(irn+irm,12)
      ncv = ncvt(lcent)
      jcv = jcvt(lcent)
c
      do 100 icv=1,ncv,jcv
         irx1 = mod(irx+icenv(1,icv),12)
         iry1 = mod(iry+icenv(2,icv),12)
         irz1 = mod(irz+icenv(3,icv),12)
c
c........does this pair make a 1bar?
c
         m2z = m2
c
c........yes
c
         if ( rt(5,1,n)+rt(5,1,m2).eq.0 ) m2z=1
c
c........no
c
         if ( rt(3,3,n)+rt(3,3,m2z).le.0 ) irz1=0
c
c........is this an operator operating along the face diagonal?
c
         if ( laue.le.3.or.m.ne.4 ) then
c
c...........no
c
            if ( rt(1,1,n)+rt(1,1,m2z).le.0 ) irx1=0
            if ( rt(2,2,n)+rt(2,2,m2z).le.0 ) iry1=0
         else
c
c...........yes
c
            irx1 = mod(irx1+iry1,12)
            iry1 = 0
         endif
         tottr = 144*irx1+12*iry1+irz1
         if ( tottr.eq.0 ) return
 100  continue
c
c.....An error has been found
c
      if (lpt.gt.0 ) write (lpt,2990) rt(5,2,n),rt(5,2,m2),
     &    tottr,irx,iry,irz,rt(5,1,n),rt(5,1,m2)
      ier = 18
      if ( lpt.gt.0 ) write (lpt,2991) m,n,m2
c
c.....formats
c
2990  format (3f10.1, 3i5, 2f10.1)
2991  format('operator ',i2,' generates matrix ',i3,' which has a',
     &       ' translation conflict ',2i3)
c
      return
      end
