c
c-----------------------------------------------------------------------
c
      subroutine steepest (uout, factor, msteps)
c
c.....steepest - multidimensional steepest descent minimization of the
c     electrostatic energy of a collection of +1 point charges lying on
c     the surface of a sphere of radius 1.
c
      include           'implicit.inc'
      include           'polyhedron.inc'
c
      integer           uout, msteps
      real*8            factor
c
      integer           istep, i, ii, k, nvar
      logical           done
      real*8            rr, sumgrad, energ
      real*8            d(0:3), y(0:3), fac, dopt, yopt
      real*8            var(mvert+mvert), grad(mvert+mvert)
      real*8            vartmp(mvert+mvert)
      real*8            gradient, energy
c
c.....Determine angular coordinates of the starting points:
c
      nvar = nvert + nvert
      do i = 1, nvert
         ii = nvert + i
         rr = sqrt(xvert(i)*xvert(i) + yvert(i)*yvert(i)
     &           + zvert(i)*zvert(i))
         var(i) = acos(zvert(i)/rr)
         var(ii) = atan2(yvert(i),xvert(i))
      enddo
c
      write (uout,100)
c
      d(1) = factor
      done = .false.
      istep = 0
      do while (.not. done)
         istep = istep + 1
c
c........Determine gradient vector with respect to theta and phi:
c
         sumgrad = gradient (var, grad, nvar, sumgrad, energ)
         sumgrad = sqrt(sumgrad)
c
c........Optimize the displacement factor:
c
         fac = -d(1) / sumgrad
         do i = 1, nvar
            vartmp(i) = var(i) + fac * grad(i)
         enddo
         y(1) = energy (vartmp, nvar)
c
         d(0) = 0.6*d(1)
         fac = -d(0) / sumgrad
         do i = 1, nvar
            vartmp(i) = var(i) + fac * grad(i)
         enddo
         y(0) = energy (vartmp, nvar)
c
         d(2) = 1.2*d(1)
         fac = -d(2) / sumgrad
         do i = 1, nvar
            vartmp(i) = var(i) + fac * grad(i)
         enddo
         y(2) = energy (vartmp, nvar)
c
         call parabola (d(0), y(0), d(1), y(1), d(2), y(2), d(3), y(3))
         fac = -d(3) / sumgrad
         do i = 1, nvar
            vartmp(i) = var(i) + fac * grad(i)
         enddo
         y(3) = energy (vartmp, nvar)
         k = 0
         if (y(1).lt.y(k)) k = 1
         if (y(2).lt.y(k)) k = 2
         if (y(3).lt.y(k)) k = 3
         yopt = y(k)
         dopt = d(k)
c
         write (uout,110) nvert, istep, sumgrad, k, dopt, yopt
         d(1) = dopt
         fac = -d(1) / sumgrad
         do i = 1, nvar
            var(i) = var(i) + fac * grad(i)
         enddo
c
         done = istep.ge.msteps .or. sumgrad.lt.1d-14
      enddo
c
c.....Store final vertices:
c
      do i = 1, nvert
         xvert(i) = radius * sthcph(i)
         yvert(i) = radius * sthsph(i)
         zvert(i) = radius * cth(i)
      enddo
c
 100  format (//
     & 1x, '+----- Minimization of the electrostatic energy -----+'//
     & 1x, '-points--step---sumgrad----k-----dopt--------------energy',
     &     '-------')
 110  format (1x, i7, i6, 1p, e12.5, 0p, i3, 1p, e12.5, e25.14)
c
      end
