c
c-----------------------------------------------------------------------
c
      subroutine writepov (file, title, uout)
c
c.....writepov - write the polyhedron description as an POV-Ray file.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'polyhedron.inc'
      include           'molge.inc'
      include           'normals.inc'
      include           'pov.inc'
      include           'strtools.inc'
c
      character*(*)     file, title
      integer           uout
c
      real*8            EPSILON
      parameter         (EPSILON = 0.01d0)
      character*(6)     color, nametext(0:MAXTEXT)
      integer           luo, i, i1, i2, i3, isum, j, j1, k, k1, k2, k3
      integer           itex
      real*8            xsc, ysc, zsc
      real*8            xct, xmin, xmax, yct, ymin, ymax
      real*8            zct, zmin, zmax, deg2rad, visdis, amplitude
      real*8            xtr, ytr, ztr, xdel, ydel, zdel, del
      real*8            x0, y0, z0, dd, ss
      real*8            xi, yi, zi, ri, xj, yj, zj
      real*8            fmin, fmax, fdel, ff
      real*8            cred, cgrn, cblu
      integer           leng, fopen
c
c.....Open the output file:
c
      if (fopen (luo, file, iowrite) .eq. iowrite) then
         call error ('writepov', 'I/O error when opening pov file',
     &               warning)
         return
      endif
      deg2rad = atan(1d0)/45d0
c
c.......................................................................
c.......................................................................
c.....Determine the gravity center of points and the extension:.........
c.......................................................................
c.......................................................................
c
      xct = 0d0
      yct = 0d0
      zct = 0d0
      if (nvert .gt. 0) then
         xmin = xvert(1)
         xmax = xvert(1)
         ymin = yvert(1)
         ymax = yvert(1)
         zmin = zvert(1)
         zmax = zvert(1)
      else if (gnpt .gt. 0) then
         xmin = gxpt(1)
         xmax = gxpt(1)
         ymin = gypt(1)
         ymax = gypt(1)
         zmin = gzpt(1)
         zmax = gzpt(1)
      else
         call error ('writepov', 'Empty graphical elements lists!',
     &               warning)
         return
      endif
c
      do i = 1, nvert
         if (xvert(i) .lt. xmin) xmin = xvert(i)
         if (xvert(i) .gt. xmax) xmax = xvert(i)
         if (yvert(i) .lt. ymin) ymin = yvert(i)
         if (yvert(i) .gt. ymax) ymax = yvert(i)
         if (zvert(i) .lt. zmin) zmin = zvert(i)
         if (zvert(i) .gt. zmax) zmax = zvert(i)
      enddo
c
      do i = 1, gnpt
         if (gxpt(i) .lt. xmin) xmin = gxpt(i)
         if (gxpt(i) .gt. xmax) xmax = gxpt(i)
         if (gypt(i) .lt. ymin) ymin = gypt(i)
         if (gypt(i) .gt. ymax) ymax = gypt(i)
         if (gzpt(i) .lt. zmin) zmin = gzpt(i)
         if (gzpt(i) .gt. zmax) zmax = gzpt(i)
      enddo
c
      xct = 0.5d0 * (xmin + xmax)
      yct = 0.5d0 * (ymin + ymax)
      zct = 0.5d0 * (zmin + zmax)
      xdel = xmax - xmin
      ydel = ymax - ymin
      zdel = zmax - zmin
c
c.....Determine the scale parameters for the x, y, and z coordinates:
c
      if (equalscale .eq. scale_all) then
c
c........The object will be
c        normalized to the range (-1,1) in the largest direction, and
c        to the appropriate length in the other directions. The geometry
c        center, on the other hand, will be translated to the origin
c        of coordinates.
c
         del = 2d0 / max(xdel,ydel,zdel)
         xsc = del
         ysc = del
         zsc = del
         xtr = xct
         ytr = yct
         ztr = zct
         amplitude = 1d0
c
      else if (equalscale .eq. scale_xy) then
c
c........The variables will try to be scaled to the range [-1,1].
c        The x and y variables will receive the same scale factor.
c
         del = 2d0 / max(xdel,ydel)
         xsc = del
         ysc = del
         zsc = 2d0 / zdel
         xtr = xct
         ytr = yct
         ztr = zct
         amplitude = 1d0
c
      else if (equalscale .eq. scale_xz) then
c
c........The variables will try to be scaled to the range [-1,1].
c        The x and z variables will receive the same scale factor.
c
         del = 2d0 / max(xdel,zdel)
         xsc = del
         ysc = 2d0 / ydel
         zsc = del
         xtr = xct
         ytr = yct
         ztr = zct
         amplitude = 1d0
c
      else if (equalscale .eq. scale_yz) then
c
c........The variables will try to be scaled to the range [-1,1].
c        The y and z variables will receive the same scale factor.
c
         del = 2d0 / max(zdel,ydel)
         xsc = 2d0 / xdel
         ysc = del
         zsc = del
         xtr = xct
         ytr = yct
         ztr = zct
         amplitude = 1d0
c
      else if (equalscale .eq. scale_none) then
c
c........The variables will be independently scaled to the range [-1,1].
c
         xsc = 2d0 / xdel
         ysc = 2d0 / ydel
         zsc = 2d0 / zdel
         xtr = xct
         ytr = yct
         ztr = zct
         amplitude = 1d0
c
      else if (equalscale .eq. scale_noscale) then
c
c........No scaling will be applied:
c
         xsc = 1d0
         ysc = 1d0
         zsc = 1d0
         xtr = 0d0
         ytr = 0d0
         ztr = 0d0
         amplitude = max(xdel, ydel, zdel)
      endif
c
c.......................................................................
c.......................................................................
c.....Write the POV file head:..........................................
c.......................................................................
c.......................................................................
c
      if (fullpov) then 
         write (luo,300)
         write (luo,301) title(1:leng(title))
         if (equalscale .eq. scale_noscale) then
            write (luo,306) xct, yct, zct, xdel, ydel, zdel
         else
            write (luo,305) xct, yct, zct, xdel, ydel, zdel
         endif
         write (luo,310) def_texture(1:leng(def_texture))
         if (putaxes) write (luo,320)
         if (mirrors) then
            write (luo,325)
            write (luo,326) 'x', mirrx, 'x'
            write (luo,326) 'y', mirry, 'y'
            write (luo,326) 'z', mirrz, 'z'
            write (luo,327) 1d2, mirry, mirrz, -1d2, mirry, mirrz, 0.02
            write (luo,327) mirrx, 1d2, mirrz, mirrx, -1d2, mirrz, 0.02
            write (luo,327) mirrx, mirry, 1d2, mirrx, mirry, -1d2, 0.02
         endif
      else
         write (luo,301) title(1:leng(title))
         if (equalscale .eq. scale_noscale) then
            write (luo,306) xct, yct, zct, xdel, ydel, zdel
         else
            write (luo,305) xct, yct, zct, xdel, ydel, zdel
         endif
      endif
 300  format (
     & '#include "colors.inc"'/
     & '#include "textures.inc"'/
     & '#include "stones.inc"'/)
 301  format (
     & '// Polyhedron created by TESSEL:'/
     & '// (c) 1996 V. Lua~na, Departamento de Quimica Fisica'/
     & '//     Universidad de Oviedo, 33006-Oviedo, Spain'/
     & '//     victor@hobbit.quimica.uniovi.es'/
     & '//'/
     & '// ', a)
 305  format (
     & '// The figure will be centered on the origin of coordinates'/
     & '// and normalized to the range (-1,1) in the largest direction'/
     & '// Before that transformation, the geometry was:'//
     & '// Figure center of gravity: ', 3f10.5/
     & '// Extension: ', 3f10.5 )
 306  format (
     & '// Figure center of gravity: ', 3f10.5/
     & '// Extension: ', 3f10.5 )
 310  format (/'#default { texture {', a, '}}'/)
 320  format (/
     & '#declare xrot= union {'/
     & '   cylinder {<0,-1,1>,<0,1,-1>,0.2}'/
     & '   cylinder {<0,-1,-1>,<0,1,1>,0.2}'/
     & '   pigment {Red}'/
     & '   scale 0.2 translate <1,0,0>'/
     & '   }'/
     & '#declare yrot= union {'/
     & '   cylinder {<0,0,0>,<1,0,1>,0.2}'/
     & '   cylinder {<0,0,0>,<-1,0,1>,0.2}'/
     & '   cylinder {<0,0,0>,<0,0,-1>,0.2}'/
     & '   pigment {Green}'/
     & '   scale 0.2 translate <0,1,0>'/
     & '   }'/
     & '#declare zrot= union {'/
     & '   cylinder {<-1,-1,0>,<-1,1,-0>,0.2}'/
     & '   cylinder {<-1,1,0>,<1,-1,-0>,0.2}'/
     & '   cylinder {<1,-1,0>,<1,1,-0>,0.2}'/
     & '   pigment {Cyan}'/
     & '   scale 0.2 translate <0,0,1>'/
     & '   }'/
     & '#declare axes = union {'/
     & '   sphere {<1,0,0>,0.04 pigment {Red}}'/
     & '   cylinder {<0,0,0>,<1,0,0>,0.04  pigment {Red}}'/
     & '   xrot'/
     & '   sphere {<0,1,0>,0.04 pigment {Green}}'/
     & '   cylinder {<0,0,0>,<0,1,0>,0.04  pigment {Green}}'/
     & '   yrot'/
     & '   sphere {<0,0,1>,0.04 pigment {Cyan}}'/
     & '   cylinder {<0,0,0>,<0,0,1>,0.04  pigment {Cyan}}'/
     & '   zrot'/
     & '   }'/
     & 'object {axes rotate <0,0,0>}')
 325  format (/
     & '#declare pln_colorx = pigment {color rgb <1.0,0.8,0.8>}'/
     & '#declare pln_colory = pigment {color rgb <0.8,1.0,0.8>}'/
     & '#declare pln_colorz = pigment {color rgb <0.8,0.8,1.0>}'/
     & '#declare pln_finish = finish {ambient 0 diffuse 0.2 ',
     &   'specular 0.1 roughness 0.002 reflection 1.0}')
 326  format (
     & 'object {plane {',a,',',f9.4,' pigment{pln_color',a,
     & '} finish{pln_finish}}',
     & ' no_shadow}'
     & )
 327  format (
     & 'object {cylinder {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & 1x, 'pigment {color rgb <0.5,0.1,0>}}}' )
c
c.......................................................................
c
      if (nvert .le. 0) GOTO 2
c
c.......................................................................
c.......................................................................
c.....Write the tessel/parametric object:...............................
c.......................................................................
c.......................................................................
c
c.....Write faces as smooth triangles:
c
 1    continue
      write (luo,330) obj_name(1:leng(obj_name))
      if (color_mode .eq. color_binary) then
c
c.......Use RED/GREEN colors depending on the sign of the face vertices:
c
        do i = 1, nface(isw1)
          i1 = iface(1,i,isw1)
          i2 = iface(2,i,isw1)
          i3 = iface(3,i,isw1)
          isum = 0
          if (fun_value(i1) .lt. 0d0)  isum = isum + 1
          if (fun_value(i2) .lt. 0d0)  isum = isum + 1
          if (fun_value(i3) .lt. 0d0)  isum = isum + 1
          if (isum.gt.1) then
             color = 'Green'
          else
             color = 'Red'
          endif
          write (luo,336)
     &      (xvert(i1)-xtr)*xsc,(yvert(i1)-ytr)*ysc,(zvert(i1)-ztr)*zsc,
     &               vnormal(1,i1), vnormal(2,i1), vnormal(3,i1),
     &      (xvert(i2)-xtr)*xsc,(yvert(i2)-ytr)*ysc,(zvert(i2)-ztr)*zsc,
     &               vnormal(1,i2), vnormal(2,i2), vnormal(3,i2),
     &      (xvert(i3)-xtr)*xsc,(yvert(i3)-ytr)*ysc,(zvert(i3)-ztr)*zsc,
     &               vnormal(1,i3), vnormal(2,i3), vnormal(3,i3),
     &      color
        enddo
        write (luo,345)
        if (fullpov) write (luo,346) obj_name(1:leng(obj_name))
c
      else if (color_mode .eq. color_hue) then
c
c.......Use the function value at the vertices to define the hue.
c       Saturation and value will be 1 by default.
c
        do i = 1, nface(isw1)
          i1 = iface(1,i,isw1)
          i2 = iface(2,i,isw1)
          i3 = iface(3,i,isw1)
          ff = (fun_value(i1)+fun_value(i2)+fun_value(i3)) / 3d0
          ff = abs(ff)
          call hsv2rgb (ff, 1d0, 1d0, cred, cgrn, cblu)
*mdc*if debug
          write (uout,*) 'writepov(debug1) --- (h;r,g,b) ', ff,
     &                   cred, cgrn, cblu
*mdc*endif
          write (luo,337)
     &     (xvert(i1)-xtr)*xsc,(yvert(i1)-ytr)*ysc,(zvert(i1)-ztr)*zsc,
     &               vnormal(1,i1), vnormal(2,i1), vnormal(3,i1),
     &     (xvert(i2)-xtr)*xsc,(yvert(i2)-ytr)*ysc,(zvert(i2)-ztr)*zsc,
     &               vnormal(1,i2), vnormal(2,i2), vnormal(3,i2),
     &     (xvert(i3)-xtr)*xsc,(yvert(i3)-ytr)*ysc,(zvert(i3)-ztr)*zsc,
     &               vnormal(1,i3), vnormal(2,i3), vnormal(3,i3),
     &     cred, cgrn, cblu
        enddo
        write (luo,345)
        if (fullpov) write (luo,346) obj_name(1:leng(obj_name))
c
      else if (color_mode .eq. color_none) then
c
c.......Give no specific color to each face. A general texture will
c       then be used for the surface:
c
        do i = 1, nface(isw1)
          i1 = iface(1,i,isw1)
          i2 = iface(2,i,isw1)
          i3 = iface(3,i,isw1)
          write (luo,335)
     &     (xvert(i1)-xtr)*xsc,(yvert(i1)-ytr)*ysc,(zvert(i1)-ztr)*zsc,
     &               vnormal(1,i1), vnormal(2,i1), vnormal(3,i1),
     &     (xvert(i2)-xtr)*xsc,(yvert(i2)-ytr)*ysc,(zvert(i2)-ztr)*zsc,
     &               vnormal(1,i2), vnormal(2,i2), vnormal(3,i2),
     &     (xvert(i3)-xtr)*xsc,(yvert(i3)-ytr)*ysc,(zvert(i3)-ztr)*zsc,
     &               vnormal(1,i3), vnormal(2,i3), vnormal(3,i3)
        enddo
        write (luo,340)
        if (fullpov)
     &     write (luo,341) obj_name(1:leng(obj_name)),
     &                     object_texture(1:leng(object_texture))
c
      else
c
c.......Use grey or rainbow scale colors:
c.......Determine first minimum and maximum function values:
c
        if (useminmax) then
           fmin = fun_min
           fmax = fun_max
        else
           fmin = fun_value(1)
           fmax = fun_value(1)
           do j = 2, nvert
              if (fun_value(j) .lt. fmin) fmin = fun_value(j)
              if (fun_value(j) .gt. fmax) fmax = fun_value(j)
           enddo
        endif
        fdel = 2d0 / (fmax-fmin)
        do i = 1, nface(isw1)
          i1 = iface(1,i,isw1)
          i2 = iface(2,i,isw1)
          i3 = iface(3,i,isw1)
          ff = (fun_value(i1)+fun_value(i2)+fun_value(i3)) / 3d0
          ff = (ff-fmin)*fdel-1d0
          if (color_mode .eq. color_grey) then
             call grayscale (ff, cred)
             cgrn = cred
             cblu = cred
          else
             call colorscale (ff, cred, cgrn, cblu, 1d0, 3d0, 1d0)
          endif
          write (luo,337)
     &     (xvert(i1)-xtr)*xsc,(yvert(i1)-ytr)*ysc,(zvert(i1)-ztr)*zsc,
     &               vnormal(1,i1), vnormal(2,i1), vnormal(3,i1),
     &     (xvert(i2)-xtr)*xsc,(yvert(i2)-ytr)*ysc,(zvert(i2)-ztr)*zsc,
     &               vnormal(1,i2), vnormal(2,i2), vnormal(3,i2),
     &     (xvert(i3)-xtr)*xsc,(yvert(i3)-ytr)*ysc,(zvert(i3)-ztr)*zsc,
     &               vnormal(1,i3), vnormal(2,i3), vnormal(3,i3),
     &     cred, cgrn, cblu
        enddo
        write (luo,345)
        if (fullpov) write (luo,346) obj_name(1:leng(obj_name))
c
      endif
 330  format ('#declare ', a, ' = union {')
 335  format (
     & 'smooth_triangle {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '}' )
 336  format (
     & 'smooth_triangle {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & 'pigment {', a, '}',
     & '}' )
 337  format (
     & 'smooth_triangle {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '> ',
     & 'pigment {color rgb <', f6.4, ',', f6.4, ',', f6.4, '>}',
     & '}' )
 340  format ('}'/)
 341  format ('object { ', a, ' texture {', a, '} rotate <0,0,0> }'/)
 345  format ('}'/)
 346  format ('object {', a, ' rotate <0,0,0>}')
c
c.......................................................................
c.......................................................................
c.....Write the molecular graphics elements: BALLS, STICKS, CONES ......
c.....and molecular POLYHEDRA ..........................................
c.......................................................................
c.......................................................................
c
 2    continue
      if (gnpt .le. 0) GOTO 3
c
c.......................................................................
c
c.....Create dumy names for the special molecular textures:
c
      do i = 1, min(9,MAXTEXT)
         write (nametext(i),'(a,i1)') 'mTx', i
      enddo
      do i = 10, min(99,MAXTEXT)
         write (nametext(i),'(a,i2)') 'mTx', i
      enddo
      do i = 100, min(999,MAXTEXT)
         write (nametext(i),'(a,i3)') 'mTx', i
      enddo
c
c.....Write now all textures definitions to the output file:
c
      write (luo,400) 'TBall', ball_texture(1:leng(ball_texture))
      write (luo,400) 'TStk', stick_texture(1:leng(stick_texture))
      write (luo,400) 'TPlh',
     &      polyhedron_texture(1:leng(polyhedron_texture))
      write (luo,400) 'TGlass', glass_texture(1:leng(glass_texture))
      do i = 1, MAXTEXT
         if (usepovtext(i))
     &      write (luo,400) nametext(i)(1:leng(nametext(i))),
     &                      povtext(i)(1:leng(povtext(i)))
      enddo
      write (luo,408) mol_name(1:leng(mol_name))
c
 400  format ('#declare ', a, ' = texture {', a, '}')
 408  format ('#declare ', a, ' = union {')
c
      del = (xsc + ysc + zsc) / 3d0
c
c.....write the balls:
c
      nametext(0) = 'TBall'
      do i = 1, gnball
         i1 = gballct(i)
         xi = (gxpt(i1)-xtr)*xsc
         yi = (gypt(i1)-ytr)*ysc
         zi = (gzpt(i1)-ztr)*zsc
         ri = gballrad(i)*del
         if (gballmaptype(i) .eq. map_none) then
c
c...........no texture map:
c
            itex = min( max(gballtext(i),0), MAXTEXT )
            if (gballrgbf(4,i).gt.EPSILON) then
c
c..............a partially transparent texture:
c
               if (gballtext(i).gt.0) then
                  write (luo,413) xi, yi, zi, ri, nametext(itex),
     &             gballrgbf(1,i), gballrgbf(2,i), gballrgbf(3,i),
     &             gballrgbf(4,i)
               else
                  write (luo,413) xi, yi, zi, ri, 'TGlass',
     &             gballrgbf(1,i), gballrgbf(2,i), gballrgbf(3,i),
     &             gballrgbf(4,i)
               endif
            else if (gballrgbf(4,i).lt.0d0) then
c
c..............an used defined texture:
c
               write (luo,415) xi, yi, zi, ri, nametext(itex)
            else
c
c..............an used defined texture and a pigment:
c
               write (luo,412) xi, yi, zi, ri, nametext(itex),
     &          gballrgbf(1,i), gballrgbf(2,i), gballrgbf(3,i)
            endif
         else if (gballmaptype(i) .eq. map_gif) then
c
c...........map a gif file on the ball:
c
            i2 = gballfp(i)
            i3 = index(filebuffer(i2:),null) - 1
            write (luo,414) 'gif', filebuffer(i2:i3), ri, xi, yi, zi
         else if (gballmaptype(i) .eq. map_tga) then
c
c...........map a tga file on the ball:
c
            i2 = gballfp(i)
            i3 = index(filebuffer(i2:),null) - 1
            write (luo,414) 'tga', filebuffer(i2:i3), ri, xi, yi, zi
         endif
      enddo
 410  format ('  union {')
 412  format (
     & '  sphere {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & ' texture {', a,
     & ' pigment {color rgb <',f6.4,',',f6.4,',',f6.4,'>}',
     & '}}' )
 413  format (
     & '  sphere {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & ' texture {', a,
     & ' pigment {color rgbf <',f6.4,',',f6.4,',',f6.4,',',f6.4,'>}',
     & '}}' )
 414  format (
     & '  sphere {<0,0,0>,1 ',
     & ' texture {TBall',
     & ' pigment {image_map {', a, ' "', a, '" map_type 1}}',
     & ' } scale ', f9.4,
     & ' translate <', f9.4, ',', f9.4, ',', f9.4, '> ',
     & '}' )
 415  format (
     & '  sphere {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & ' texture {', a,
     & '}}' )
c
c.....write the sticks:
c
      nametext(0) = 'TStk'
      do i = 1, gnstk
         i1 = gstkpt(1,i)
         i2 = gstkpt(2,i)
         xi = (gxpt(i1)-xtr)*xsc
         yi = (gypt(i1)-ytr)*ysc
         zi = (gzpt(i1)-ztr)*zsc
         xj = (gxpt(i2)-xtr)*xsc
         yj = (gypt(i2)-ytr)*ysc
         zj = (gzpt(i2)-ztr)*zsc
         ri = gstkrad(i)*del
         itex = min( max(gstktext(i),0), MAXTEXT)
         if (gstkdefrgb(i)) then
            write (luo,422) xi, yi, zi, xj, yj, zj, ri
         else if (gstkrgbf(4,i).gt.EPSILON) then
            if (gstktext(i).eq.0) then
               write (luo,423) xi, yi, zi, xj, yj, zj, ri, 'TGlass',
     &                         (gstkrgbf(k,i), k = 1, 4)
            else
               write (luo,423) xi, yi, zi, xj, yj, zj, ri,
     &                         nametext(itex), (gstkrgbf(k,i), k = 1, 4)
            endif
         else if (gstkrgbf(4,i).lt.0d0) then
            write (luo,425) xi, yi, zi, xj, yj, zj, ri, nametext(itex)
         else
            write (luo,424) xi, yi, zi, xj, yj, zj, ri, nametext(itex),
     &       gstkrgbf(1,i), gstkrgbf(2,i), gstkrgbf(3,i)
         endif
      enddo
 422  format (
     & '  cylinder {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & 1x, 'texture {TStk}}' )
 423  format (
     & '  cylinder {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & ' texture {', a,
     & ' pigment {color rgbf <',f6.4,',',f6.4,',',f6.4,',',f6.4,'>}',
     & '}}' )
 424  format (
     & '  cylinder {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & ' texture {', a,
     & ' pigment {color rgb <',f6.4,',',f6.4,',',f6.4,'>}',
     & '}}' )
 425  format (
     & '  cylinder {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & ' texture {', a,
     & '}}' )
c
c.....write the cones:
c
      nametext(0) = 'TStk'
      do i = 1, gncone
         i1 = gcnpt(1,i)
         i2 = gcnpt(2,i)
         xi = (gxpt(i1)-xtr)*xsc
         yi = (gypt(i1)-ytr)*ysc
         zi = (gzpt(i1)-ztr)*zsc
         xj = (gxpt(i2)-xtr)*xsc
         yj = (gypt(i2)-ytr)*ysc
         zj = (gzpt(i2)-ztr)*zsc
         ri = gcnrad(i)*del
         itex = min( max(gcntext(i),0), MAXTEXT)
         if (gcndefrgb(i)) then
            write (luo,432) xi, yi, zi, ri, xj, yj, zj
         else if (gcnrgbf(4,i).gt.EPSILON) then
            if (gcntext(i).eq.0) then
               write (luo,433) xi, yi, zi, ri, xj, yj, zj, 'TGlass',
     &                         (gcnrgbf(k,i), k = 1, 4)
            else
               write (luo,433) xi, yi, zi, ri, xj, yj, zj,
     &                         nametext(itex), (gcnrgbf(k,i), k = 1, 4)
            endif
         else if (gcnrgbf(4,i).lt.0d0) then
            write (luo,435) xi, yi, zi, ri, xj, yj, zj, nametext(itex)
         else
            write (luo,434) xi, yi, zi, ri, xj, yj, zj, nametext(itex),
     &       gcnrgbf(1,i), gcnrgbf(2,i), gcnrgbf(3,i)
         endif
      enddo
 432  format (
     & '  cone {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, 0.0',
     & 1x, 'texture {TStk}}' )
 433  format (
     & '  cone {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, 0.0',
     & ' texture {', a,
     & ' pigment {color rgbf <',f6.4,',',f6.4,',',f6.4,',',f6.4,'>}',
     & '}}' )
 434  format (
     & '  cone {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, 0.0',
     & ' texture {', a,
     & ' pigment {color rgb <',f6.4,',',f6.4,',',f6.4,'>}',
     & '}}' )
 435  format (
     & '  cone {',
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, ', f9.4,
     & '<', f9.4, ',', f9.4, ',', f9.4, '>, 0.0',
     & ' texture {', a,
     & '}}' )
c
c.....write the molecular polyhedra:
c
      nametext(0) = 'TPlh'
      if (use_planes) then
         do i = 1, gnplh
            write (luo,442)
            i1 = gctplh(i)
            x0 = gxpt(i1)
            y0 = gypt(i1)
            z0 = gzpt(i1)
            do j = 1, gnface(i)
               j1 = giface(j,i)
               dd = (gnormal(1,j1)*x0 + gnormal(2,j1)*y0 +
     &               gnormal(3,j1)*z0 + gnormal(4,j1))
               dd = dd * xsc
               write (luo,444) -gnormal(1,j1), -gnormal(2,j1), 
     &                         -gnormal(3,j1), dd
            enddo
            x0 = (gxpt(i1)-xtr)*xsc
            y0 = (gypt(i1)-ytr)*ysc
            z0 = (gzpt(i1)-ztr)*zsc
            itex = min( max(gplhtext(i),0), MAXTEXT)
            write (luo,446) x0, y0, z0
            if (gplhrgbf(4,i).gt.EPSILON) then
               if (gplhtext(i).eq.0) then
                  write (luo,449) 'TGlass', (gplhrgbf(j,i), j = 1, 4)
               else
                  write (luo,449) nametext(itex), (gplhrgbf(j,i), j=1,4)
               endif
            else if (gplhrgbf(4,i).lt.0d0) then
               write (luo,447) nametext(itex)
            else
               write (luo,448) nametext(itex), (gplhrgbf(j,i), j = 1,3)
            endif
         enddo
      else
         do i = 1, gnplh
            write (luo,452)
            do j = 1, gnface(i)
               j1 = giface(j,i)
               k1 = giside(1,j1)
               do k = 2, gnside(j1)-1
                  k2 = giside(k,j1)
                  k3 = giside(k+1,j1)
                  write (luo,454)
     &               (gxpt(k1)-xtr)*xsc, (gypt(k1)-ytr)*ysc,
     &               (gzpt(k1)-ztr)*zsc,
     &               (gxpt(k2)-xtr)*xsc, (gypt(k2)-ytr)*ysc,
     &               (gzpt(k2)-ztr)*zsc,
     &               (gxpt(k3)-xtr)*xsc, (gypt(k3)-ytr)*ysc,
     &               (gzpt(k3)-ztr)*zsc
               enddo
            enddo
            itex = min( max(gplhtext(i),0), MAXTEXT)
            if (gplhrgbf(4,i).gt.EPSILON) then
               if (gplhtext(i).eq.0) then
                  write (luo,459) 'TGlass', (gplhrgbf(j,i), j = 1, 4)
               else
                  write (luo,459) nametext(itex), (gplhrgbf(j,i), j=1,4)
               endif
            else if (gplhrgbf(4,i).lt.0d0) then
               write (luo,457) nametext(itex)
            else
               write (luo,458) nametext(itex), (gplhrgbf(j,i), j = 1,3)
            endif
         enddo
      endif
c
 442  format (
     & '  intersection {')
 444  format ('    ',
     & 'plane {<', f10.4, ',', f10.4, ',', f10.4, '>,', f10.4, '}')
 446  format ('    ',
     & 'translate <', f10.4, ',', f10.4, ',', f10.4, '>')
 447  format ('    ',
     & 'texture {', a,
     & '  }')
 448  format ('    ',
     & 'texture {', a,
     & ' pigment {color rgb <',f6.4,',',f6.4,',',f6.4,'>}}'/
     & '  }')
 449  format ('    ',
     & 'texture {', a,
     & ' pigment {color rgbf <',f6.4,',',f6.4,',',f6.4,',',f6.4,'>}}'/
     & '  }')
 452  format ('  union {')
 454  format ('    triangle {<',
     &      f10.4, ',', f10.4, ',', f10.4, '>,',
     & '<', f10.4, ',', f10.4, ',', f10.4, '>,',
     & '<', f10.4, ',', f10.4, ',', f10.4, '> }')
 457  format (
     & '    texture {', a,
     & '    }')
 458  format (
     & '    texture {', a,
     & ' pigment {color rgb <',f6.4,',',f6.4,',',f6.4,'>}}'/
     & '    }')
 459  format (
     & '    texture {', a,
     & ' pigment {color rgbf <',f6.4,',',f6.4,',',f6.4,',',f6.4,'>}}'/
     & '    }')
c
      write (luo,480)
 480  format ('}'/)
c
c.....create the final molecular object:
c
      if (fullpov) write (luo,490) mol_name(1:leng(mol_name))
 490  format ('object {', a, ' rotate <0,0,0>}')
c
c.......................................................................
c.......................................................................
c.....Add a camera, some light sources and a background:................
c.......................................................................
c.......................................................................
c
 3    continue
      if (fullpov) then
c
c........Calculate the camera parameters: position and look at:
c
         if (usecamangle) then
            visdis = zoom * amplitude / tan(camangle(3)*deg2rad/2d0)
            ss = sin(camangle(1)*deg2rad)
            COP(1) = visdis * ss * cos(camangle(2)*deg2rad)
            COP(2) = visdis * ss * sin(camangle(2)*deg2rad)
            COP(3) = visdis * cos(camangle(1)*deg2rad)
            VRP(1) = xct - xtr
            VRP(2) = yct - ytr
            VRP(3) = zct - ztr
c
            write (luo,501) COP(1), COP(2), COP(3),
     &                      SKY(1), SKY(2), SKY(3),
     &                      VUV(1), VUV(2), VUV(3),
     &                      RHT(1), RHT(2), RHT(3),
     &                      DRT(1)*zoom, DRT(2)*zoom, DRT(3)*zoom,
     &                      VRP(1), VRP(2), VRP(3)
         else
c
            write (luo,501) COP(1), COP(2), COP(3),
     &                      SKY(1), SKY(2), SKY(3),
     &                      VUV(1), VUV(2), VUV(3),
     &                      RHT(1), RHT(2), RHT(3),
     &                      DRT(1), DRT(2), DRT(3),
     &                      VRP(1), VRP(2), VRP(3)
         endif
c
c........Put a light source at the camera and another in the sky:
c
         if (def_light) then
            write (luo,511) COP(1), COP(2), COP(3), 'White'
            write (luo,511) SKY(1)*1d2, SKY(2)*1d2, SKY(3)*1d2, 'White'
         endif
         do i = 1, n_light
            if (use_light(i)) then
               x0 = r_light(i) * sin(theta_light(i)*deg2rad)
     &                         * cos(phi_light(i)*deg2rad)
               y0 = r_light(i) * sin(theta_light(i)*deg2rad)
     &                         * sin(phi_light(i)*deg2rad)
               z0 = r_light(i) * cos(theta_light(i)*deg2rad)
               write (luo,511) x0, y0, z0,
     &                         col_light(i)(1:leng(col_light(i)))
            endif
         enddo
c
c........Add a default background:
c
         write (luo,520) def_background(1:leng(def_background))
c
c........Put additional elements like sky, floor, etc:
c
         if (sky_type .eq. 0) then
            continue
         else if (sky_type .eq. -1) then
            write (luo,530) sky_file(1:leng(sky_file))
         endif
         if (floor_type .eq. 0) then
            continue
         else if (floor_type .eq. -1) then
            write (luo,530) floor_file(1:leng(floor_file))
         endif
         if (wall_type .eq. 0) then
            continue
         else if (wall_type .eq. -1) then
            write (luo,530) wall_file(1:leng(wall_file))
         endif
c
      endif
c
 501  format (
     & 'camera {'/
     & '  location <', f10.5, ',', f10.5, ',', f10.5, '>'/
     & '  sky   <', f5.2, ',', f5.2, ',', f5.2, '>'/
     & '  up    <', f5.2, ',', f5.2, ',', f5.2, '>'/
     & '  right <', f5.2, ',', f5.2, ',', f5.2, '>'/
     & '  direction <', f10.5, ',', f10.5, ',', f10.5, '>'/
     & '  look_at <', f5.2, ',', f5.2, ',', f5.2, '>'/
     & '}' )
 511  format (/
     & 'light_source {<', f10.5, ',', f10.5, ',', f10.5, '> color ', a,
     & '}')
 520  format (a)
 530  format (/'#include "', a, '"')
c
      call fclose (luo)
c
c.....Activity report:
c
      write (uout,900) file(1:leng(file)), nvert, nvf, nface(isw1),
     &                 gnball, gnstk, gncone, gnplh
 900  format (/
     & 1x, 'WRITEPOV: writing a description of the object in the POV'/
     & 1x, ' format to be plot and manipulated with POV-Ray 2.2.'//
     & 1x, 'Name of the POV file:        ', a/
     & 1x, '-- POLYHEDRAL OBJECT --'/
     & 1x, 'Number of vertices created:  ', i6/
     & 1x, 'Number of vertices per face: ', i6/
     & 1x, 'Number of faces:             ', i6/
     & 1x, '-- MOLECULAR OBJECT --'/
     & 1x, 'Number of balls:     ', i6/
     & 1x, 'Number of sticks:    ', i6/
     & 1x, 'Number of cones:     ', i6/
     & 1x, 'Number of polyhedra: ', i6)
c
      end
